# クラス
module DarkHall
	module InstanceFromModel
		attr_reader :data_symbol
		def initialize(data_symbol)
			super()
			@data_symbol = data_symbol.to_sym
		end
		
		def data
			raise
		end
		
		def method_missing(message, *arg)
	    self.data.send(message, *arg)
		end
		
		
	end
	
	class ModelInstance
		include InstanceFromModel
	end
	
	class Model
		private
		def initialize(parameters = {})
			init_params
			return unless parameters
			parameters.each_pair do |key, value|
				instance_variable_set("@#{key.to_s}", value)
			end
		end
		
		def init_params
			
		end	
	end


		DIR_N = 0
		DIR_W = 3
		DIR_E = 1
		DIR_S = 2
	
		
	
	
	class Item < StorableObject
		include InstanceFromModel
		attr_reader :unidentified
		alias unidentified? unidentified

		def initialize(data_symbol)
			super(data_symbol)
			@unidentified = false
		end
		def identify
			@unidentified = false
		end
		
		def unidentify
			@unidentified = true if self.black_name
		end
		
		def price
			return data.price.to_i
		end
		
		def sell_price
			return (unidentified? ? 0 : data.sell_price.to_i)
		end
		
		def sell_price
			return (unidentified? ? 0 : data.sell_price.to_i)
		end

		
		def description
			return (unidentified? ? "(未識別)" : data.description)
		end

		
		def name
			(unidentified ? self.black_name : self.data.name)
		end
		
		def usable_by?(member)
			(unidentified? ? false : super)
		end
		
		

		def data
			return ITEM_DATA[@data_symbol]
		end
		

		
		def dump
			hash = super
			hash[:data_symbol] = @data_symbol
			hash[:unidentified] = @unidentified
			return hash
		end
		
		def self.json_create(hash)
			obj =  new(hash['data_symbol'])
			obj.unidentify if hash['unidentified']
			return obj
		end
	end
	
	class BundleItem < Item
		attr_accessor :number
		
		def initialize(data_symbol, number = 1)
			super(data_symbol)
			@number = number
		end	
		
		def price
			return (data.price * @number).to_i
		end
		
		def sell_price
			return (data.sell_price * @number.to_f).to_i
		end

		
		def name
			return "#{@number}#{self.unit}の#{super}"
		end
		
		def dump
			hash = super
			hash[:number] = @number
			return hash
		end
		
		def self.json_create(hash)
			return new(hash['data_symbol'], hash['number'])
		end

	end
	
	
	class Equipment < Item
		attr_accessor :prefix
	
		def initialize(data_symbol, prefix = :Standard)
			super(data_symbol)
			@prefix = prefix
		end	
		
		def name
			if Lang::Prefix.const_defined?(@prefix) then
				return(Lang::Prefix.const_get(@prefix) + self.data.name)
			else
				return super
			end
		end
		
		def extra_captions
			captions = super.dup
			case @prefix
			when :Master
				captions << "熟練限界2倍"
			end
			
			return captions
		end
		
		def price
			price = self.data.price
			case @prefix
			when :Broken
				return(price / 5)
			when :Bad
				return (price / 2)
			when :Good
				return (price * 1.5).to_i

			when :Gorgeous
				return(price * 5)
			when :Master
				return(price * 10)
			else
				return price
			end
		end
		
		
		PREFIX_TABLE = []
		PREFIX_TABLE[0] = []
		PREFIX_TABLE[0].fill(:Broken, 0...4)
		PREFIX_TABLE[0].fill(:Bad, 4...10)
		PREFIX_TABLE[0].fill(:Standard, 10...20)

		PREFIX_TABLE[1] = []
		PREFIX_TABLE[1].fill(:Broken, 0...2)
		PREFIX_TABLE[1].fill(:Bad, 2...6)
		PREFIX_TABLE[1].fill(:Standard, 6...16)
		PREFIX_TABLE[1].fill(:Good, 16...20)

		PREFIX_TABLE[8] = []
		PREFIX_TABLE[8] << :Bad
		PREFIX_TABLE[8] << :Standard
		PREFIX_TABLE[8] << :Good
		PREFIX_TABLE[8] << :Sophisticated
		PREFIX_TABLE[8] << :Hard
		PREFIX_TABLE[8] << :Sharp
		PREFIX_TABLE[8] << :Gorgeous
		PREFIX_TABLE[8] << :Master
		
		def set_prefix(rarity = 0)
			10.times do
				@prefix = PREFIX_TABLE[rarity][rand(PREFIX_TABLE[rarity].size)]
				next unless self.matched_prefix?(@prefix)
				return self
			end
			
			@prefix = :Standard
			return self
		end
		
		def dump
			hash = super
			hash[:prefix] = @prefix
			return hash
		end
		
		def self.json_create(hash)
			obj = super
			obj.prefix = hash['prefix'].to_sym

			return obj
		end

	end
	
	class Weapon < Equipment
	
		def data
			return WEAPON_DATA[@data_symbol]
		end
	
		def damage_rate
			case @prefix
			when :Sharp
				return data.damage_rate * 1.4
			when :Good
				return data.damage_rate * 1.2
			when :Bad
				return data.damage_rate * 0.8
			when :Broken
				return data.damage_rate * 0.5
			else
				return data.damage_rate
			end
		end
		
		def power(actor)
			params = data.attack_parameters
			value = 0
			params.each do |param|
				value += (actor.get_ability(param) / params.size)
			end
			return(value * self.damage_rate).to_i
		end
		
		alias attack power
		
		def hitting_bonus
			case @prefix
			when :Sophisticated
				return data.hitting_bonus + 3
			when :Good
				return data.hitting_bonus + 1
			when :Bad
				return data.hitting_bonus - 1
			when :Broken
				return data.hitting_bonus - 2
			else
				return data.hitting_bonus
			end
		end
		
		
	
		HAND = Weapon.new(:Hand)
	
	end

	
	class Armor < Equipment
	
		def data
			return ARMOR_DATA[@data_symbol]
		end
	
	end

	class Accessory < Equipment
	
		def data
			return ACCESSORY_DATA[@data_symbol]
		end
	
	end

end
	
