# ダンジョンを表すクラス


module DarkHall


	module Dungeon
		module Area
			WALL = 0
			ROOM = 1
			ROUTE = 2
			DOOR = 3
			UP_STAIR = 4
			DOWN_STAIR = 5
			EXIT_DOOR = 6
		end		


		class Floor
			attr_accessor :name
			attr_accessor :area_data, :room_data, :room_max, :region_data, :room_to_region_table
			attr_accessor :width, :height
			attr_accessor :found_area
			attr_accessor :tereport_points
			attr_accessor :special_action_data
			
			
			def initialize
				@name = "1階"
				@width = 60
				@height = 60
				@room_max = nil
				@room_to_region_table = []
				@area_data = Array.new(@width * 2 + 1)
				@area_data.each_index{|i| @area_data[i] = Array.new(@height * 2 + 1)}
				@area_data.each{|x| x.fill(Area::WALL)}
				@room_data = Array.new(@width)
				@room_data.each_index{|i| @room_data[i] = Array.new(@height)}
				@region_data = Array.new(@width)
				@region_data.each_index{|i| @region_data[i] = Array.new(@height)}
				@tereport_points = []
				@target_region_id = nil
				
				@special_action_data = []
			end
			
			def [](half_x, half_y)
				return @area_data[half_x][half_y]
			end
			
			def []=(half_x, half_y, element)
				@area_data[half_x][half_y] = element
			end
			
			def flooring(x, y)
				return self[x * 2 + 1, y * 2 + 1]
			end
			
			def set_flooring(x, y, area)
				self[x * 2 + 1, y * 2 + 1] = area
			end
			
			def wall(direction, x, y)
				case direction
				when DIR_N
					return self[x * 2 + 1, y * 2]
				when DIR_E
					return self[x * 2 + 1 + 1, y * 2 + 1]
				when DIR_S
					return self[x * 2 + 1, y * 2 + 1 + 1]
				when DIR_W
					return self[x * 2, y * 2 + 1]
				end
			end
			
			def set_wall(direction, x, y, area = Area::WALL)
				case direction
				when DIR_N
					self[x * 2 + 1, y * 2] = area
				when DIR_E
					self[x * 2 + 1 + 1, y * 2 + 1] = area
				when DIR_S
					self[x * 2 + 1, y * 2 + 1 + 1] = area
				when DIR_W
					self[x * 2, y * 2 + 1] = area
				end
				return self
			end
			
			def set_door(direction, x, y)
				set_wall(direction, x, y, Area::DOOR)
			end
			
			def in_region(id)
				@target_region_id = id
				yield
				@target_region_id = nil
			end
			
			def get_flooring_data
				data = []
				@width.each_index do |x|
					data[x] = []
					@height.each_index do |y|
						data[x][y] = @area_data[x * 2 + 1][y * 2 + 1]
					end
				end
				
				return data
			end
			
			
			
			def set_room(left, top, width, height, region_id = @target_region_id)
				set_flooring_area(Area::ROOM, left, top, width, height, region_id)
				return self
			end
			
			def set_route(left, top, width, height, region_id = @target_region_id)
				set_flooring_area(Area::ROUTE, left, top, width, height, region_id)
				return self
			end
			
			def set_tereport_point(left, top)
				@tereport_points << [left, top]
			end
			
			def set_wall_area(left, top, width, height)
				(left...(left + width)).each do |x|
					half_top = top * 2
					half_height = height * 2 + 1
					@area_data[x * 2].fill(Area::WALL, half_top, half_height)
					@area_data[x * 2 + 1].fill(Area::WALL, half_top, half_height)
					@area_data[x * 2 + 2].fill(Area::WALL, half_top, half_height)
					
				end
				return self
			end

			# 床箇所のみを指定タイプのエリアで埋める
			def set_flooring_area(area_type, left, top, width, height, region_id = nil)
				(left...(left + width)).each do |x|
					half_top = top * 2 + 1
					half_height = height * 2 - 1
					@area_data[x * 2 + 1].fill(area_type, half_top, half_height)
					@region_data[x].fill(region_id, top, height)
					# 最後のループならスキップ
					next if x == left + width - 1
					
					@area_data[x * 2 + 2].fill(area_type, half_top, half_height)
					
				end
				return self
			end
			
			# 部屋分けの更新
			def update_room_data
				@room_to_region_table.clear
				# ひとまず全部屋を-1で塗りつぶす
				@room_data.each_index do |x|
					@room_data[x].each_index do |y|
						if self.flooring(x, y) == Area::ROOM then
							@room_data[x][y] = -1
						else
							@room_data[x][y] = nil
						end
					end
				end
				
				# 番号分け
				
				id = 0
				@room_data.each_index do |x|
					@room_data[x].each_index do |y|
						if @room_data[x][y] == -1 then
							room_fill_sequence(x, y, id)
							@room_to_region_table[id] = @region_data[x][y]
							
							id += 1
						end
					end
				end
				@room_max = id
				
			end
			
			
			
			def room_fill_sequence(x, y, room_id)
				
				@room_data[x][y] = room_id
				
				# 四方をそれぞれチェック
				if regular_position?(x, y - 1) && (wall(DIR_N, x, y) == Area::ROOM) &&
				(@room_data[x][y - 1] == -1) then
					room_fill_sequence(x, y - 1, room_id)
				end
				
				if regular_position?(x + 1, y) && (wall(DIR_E, x, y) == Area::ROOM) &&
				(@room_data[x + 1][y] == -1) then
					room_fill_sequence(x + 1, y, room_id)
				end
				
				if regular_position?(x, y + 1) && (wall(DIR_S, x, y) == Area::ROOM) &&
				(@room_data[x][y + 1] == -1) then
					room_fill_sequence(x, y + 1, room_id)
				end
				
				if regular_position?(x - 1, y) && (wall(DIR_W, x, y) == Area::ROOM) &&
				(@room_data[x - 1][y] == -1) then
					room_fill_sequence(x - 1, y, room_id)
				end

			end
			
			def regular_position?(x, y)
				return((0...@width).include?(x) && (0...@height).include?(y))
			end
			
			
			
			# パーティーの視界に入る部分のエリア情報を得る
			def get_viewable_area_data(party)
				data = []
				(VIEWABLE_FRONT_RANGE * 2 + 1).times do |i|
					data[i] = {}
				end
				
				
				data.each_index do |half_range|
					((VIEWABLE_SIDE_RANGE * 2 * -1)..(VIEWABLE_SIDE_RANGE * 2)).each do |half_pos|
						data[half_range][half_pos] = get_area_data(party, half_range, half_pos)
					end
				end
				
				return data
			end
			
			def get_area_data(party, half_range, half_pos)
				half_x = party.x * 2 + 1
				half_y = party.y * 2 + 1
				case party.direction
				when DIR_N
					@area_data[half_x + half_pos][half_y - half_range]
				when DIR_E
					@area_data[half_x + half_range][half_y + half_pos]
				when DIR_S
					@area_data[half_x - half_pos][half_y + half_range]
				when DIR_W
					@area_data[half_x - half_range][half_y - half_pos]
				end
				
			end
			
			# マップ埋め
			def find(x, y)
				found_area = $found_area[$party.floor_id]
				if found_area[x][y] == 0 then
					case flooring(x, y)
					when Area::ROOM
						room_id = @room_data[x][y]
						found_area.each_index do |cx|
							found_area[x].each_index do |cy|
								found_area[cx][cy] = 1 if @room_data[cx][cy] == room_id
							end
						end
					else
						found_area[x][y] = 1
					end
				end
			end
			
			def on_enter_region(party, old_region)
			end
			
			def on_walk(party)
			end
			
			def on_move(party)
			end
			
			def on_detect(party)
			end
			
			def on_special_action(party, action_id)
			end
			


			private
			
			def show_special_action(id, caption)
				@special_action_data << [id, caption]

			end
			
			def reload_dungeon
				Dungeon.load_data
			end
			
			def save(scene = "")
				save_profile(FQ_BIG_CHANGE, scene)
			end
			
			
		end
		
	end
	include Dungeon
	


=begin
	module Dungeon

		class Floor
			attr_accessor :name
			attr_accessor :area_data
			attr_accessor :width, :height
			
			def initialize(width = 80, height = 60)
				@width = width
				@height = height
				@area_data = Array.new(@width)
				@area_data.each_index{|i| @area_data[i] = Array.new(@height)}
				@area_data.each{|x| x.fill(Area::WALL)}
				
			end
			
			def set_room(left, top, width, height)
				(left...(left + width)).each do |x|
					@area_data[x].fill(Area::FLOORING, top, height)
				end
				return self
			end
			
			# パーティーの視界に入る部分のエリア情報を得る
			def get_viewable_area_data
				data = []
				(VIEWABLE_FRONT_RANGE + 1).times do |i|
					data[i] = {}
				end
				
				
				data.each_index do |range|
					((VIEWABLE_SIDE_RANGE * -1)..VIEWABLE_SIDE_RANGE).each do |pos|
						data[range][pos] = get_area_data($party, range, pos)
					end
				end
				
				return data
			end
			
			def get_area_data(party, range, pos)
				case party.direction
				when DIR_N
					@area_data[party.x + pos][party.y - range]
				when DIR_E
					@area_data[party.x + range][party.y + pos]
				when DIR_S
					@area_data[party.x - pos][party.y + range]
				when DIR_W
					@area_data[party.x - range][party.y - pos]
				end
				
			end
		
		end
		
	end
=end

	FLOORS = []
	
	module Dungeon
		def Dungeon.load_data
			FLOORS.clear
			
			dir_path = Pathname.new('./dungeon')
			rb_pattern = /(\d+)f\.rb\Z/
			crb_pattern = /(\d+)f\.crb\Z/
			
			dir_path.children.each do |path|
				case path.to_s.downcase
				when rb_pattern
					src = path.read
				when crb_pattern
					src = DarkHall.decrypt(path.read)
				else
					next
				end
				
				floor = Floor.new
				floor.instance_eval(src, path.basename.to_s)
				floor.update_room_data
				FLOORS[$1.to_i] = floor
				LOGGER.puts "ダンジョンマップファイル #{path.to_s} ロード完了"
			end
		
		end
	end
	
end

	
	
	
	
