	#　ステータスクラス

module DarkHall

	TG_MEMBER = 1
	TG_ENEMY = 2
	TG_NONE = 3

	class Effect
	
		attr_accessor :camp_usable, :battle_usable, :town_usable
		attr_accessor :fast
		attr_accessor :messages, :self_use_messages

		def initialize(parameters = {})
			init_params
			return unless parameters
			parameters.each_pair do |key, value|
				instance_variable_set("@#{key.to_s}", value)
			end
		end
		
		def init_params
			@messages = []
			@self_use_messages = []
			@fast = false
			init_usability(false, false, false)
		end	

		
		
		def init_usability(town, camp, battle)
			@town_usable = town
			@camp_usable = camp
			@battle_usable = battle
			return self
		end
		
		def get_message(actor, targets)
			if !(@self_use_messages.empty?) && targets == [actor] then
				msgs = @self_use_messages
			else
				msgs = @messages
			end
		
			return "" if msgs.empty?
			msg = msgs[rand(msgs.size)]
			return eval_message(msg, :actor => (actor ? actor.name : nil), :target => self.target_name(targets))
		end
		
		def target_name(targets)
			return(targets.size >= 2 ? "#{targets.first.name}たち" : targets.first.name)
		end
		
		def target_type
			return TG_NONE
		end
		
		
		def operate(actor)
			return get_message(actor, nil)
		end

	end
	
	
	module MagicEffect
		def magic_attack_bonus(actor, rate)
			return (actor.magic_attack * rate).to_i
		end
		
		def magic_defense_bonus(actor, rate)
			return (actor.magic_defense * rate).to_i
		end
	end
	
	
	class SpecialEffect < Effect
		attr_accessor :proc
		def init_params
			super
			@proc = nil
		end
		
		def operate(actor)
			return catch(:continue){
				proc.call(actor)
			}
		end
	end

	# 対象メンバーを違う場所へすっ飛ばす
	class MemberTereportEffect < Effect
		
		def operate(actor, members)
			msgs = []
			msgs << get_message(actor, members)
			new_party = Party.new
			members.each do |member|
				$party.leave_member($members.index(member))
				new_party.join_member($members.index(member))
				msgs << Message.on_tereported(member)
			end
			
			
			points = $party.floor.tereport_points
			point = points[rand(points.size)]
			new_party.x = point[0]
			new_party.y = point[1]
			new_party.floor_id = $party.floor_id
			$dungeon_parties << new_party
			
			
			
			if $party.members.empty? then
				$dungeon_parties.delete($party)
				$party = new_party
				
			end
			
			PARTY_WINDOW.update
			DUNGEON_WINDOW.update
			
			return msgs
		end
	end
	
	
	# ダメージや状態異常を与える
	class AttackEffect < Effect
		attr_accessor :fixed_damage
		attr_accessor :through_defense
		attr_accessor :shield_guardable
		attr_accessor :attack_number
		attr_accessor :hitting_border
		attr_accessor :damage_types
		attr_accessor :state_data
		
		def init_params
			super
			init_usability(false, false, true)
			@fixed_damage = nil
			@damage_types = []
			@through_defense = false
			@shield_guardable = true
			@attack_number = nil
			@hitting_border = nil
			@state_data = []
		end
		
		def get_deadly_message(target)
			if target.dead? then
				return Message.on_dead(target)
			elsif target.deadly? then
				return Message.on_deadly(target)
			end
		end
				
		
		def operate(actor, targets)
			result = [get_message(actor, targets)]

			
			targets.each do |target|
				LOGGER.puts("\t→#{target.name}")
			
			
				clean = check_clean_hit(actor, target)
				
				messages = []
				if clean then
					messages << Message.on_clean_hit
					LOGGER.puts("\t\tクリーンヒット")
					hit_number = calc_hit_number(actor, target, true)
				else
					hit_number = calc_hit_number(actor, target)
				end
				
				
				if hit_number == 0 then
					result << Message.on_no_hit(target)
					LOGGER.puts("\t\t#{target.name}には命中しなかった")
					next
				elsif !(clean) && @shield_guardable && target.shield_usable? && (shield_guard = check_shield_guard(actor, target)) then

					LOGGER.puts("\t\tガード成功 (防護+#{target.shield.extra_defense})")

					messages << Message.on_shield_guard(target)
					target.on_use_shield
				end
				
				
				
				# ダメージ
				value = 0
				buf = []
				(hit_number || 1).times do |i|
					dmg = calc_damage(actor, target, clean, shield_guard)
					buf << dmg
					value += dmg
				end
				if value > 0 then
					# ダメージログ
					res = target.get_damage_resist(self.get_damage_types(actor))
					rate = RES_DAMAGE_RATE_TABLE[res]
					LOGGER.puts "\t\t耐性ランク#{res} - ダメージ#{rate}倍" unless res == RES_NORMAL
					LOGGER.puts "\t\tダメージ: #{buf.join(' + ')}"
					#target.damaged_response(self, value)
					
					target.damage(value)
					messages << Message.on_damaged(target, value, hit_number)
					s = (hit_number ? "#{hit_number}回命中し、" : "")
					LOGGER.puts("\t\t#{s}#{target.name}に#{value}のダメージ (HP:#{target.hp}/#{target.hp_max})")
					
					
					# 重傷＆死亡メッセージ
					msg = get_deadly_message(target)
					messages << msg if msg
					
					# 怯み
					if target.alive? && clean then
						target.finish_action
						messages << Message.on_recoiled(target)
					end
					
					# 状態異常チェック
					if target.alive? then
						target.status.each do |state|
							msg = state.on_damaged(target, value)
							messages << msg if msg
						end
					end
				else
					LOGGER.puts "\t\tダメージ0"
				end

				
				# 状態異常付加
				status = target.status
				@state_data.each do |state, border|
					if state.is_a?(CurseState) && $battle.praying? then
						messages << "呪いは祈りにより打ち消された"
						next
					end
					
					#border += RES_STATE_BORDER_BONUS_TABLE[target.get_state_resist(state.class.to_s.to_sym)] if border
					if border.nil? || Roll.check_3d33(border) then
						if target.add_state(state) then
							messages << state.get_added_message(target) if target.alive?
							LOGGER.puts("\t\t#{state.name}を付加 (Lv#{state.level})")
						end
					end
					
				end
				
				messages << Message.on_no_effect(target) if messages.empty?
				result << messages.join("\n")

			end
			
			
			
			return result.compact

		end
		
		
		def get_attack(actor)
			return @fixed_damage
		end
		
		def get_defense(target, shield_guard = false)
			if @through_defense then
				LOGGER.puts "\t\t防護貫通効果"
				return 0
			elsif shield_guard then
				return target.defense + target.shield.extra_defense
			else
				return target.defense
			end
			
		end
		
		# ダメージ計算
		def calc_damage(actor, target, clean_hit = false, shield_guard = false)
			if @fixed_damage || @damage_rate then
				atk = get_attack(actor)
				df = get_defense(target, shield_guard)

				
				res = target.get_damage_resist(self.get_damage_types(actor))
				rate = RES_DAMAGE_RATE_TABLE[res]
				damage = double_roll(atk, df)
				damage *= rate
				
				if target.guarding? || target.blocking? then
					damage *= 0.7
					LOGGER.puts "\t\t防御状態のためダメージ0.7倍"
				end
				return [damage.to_i, 1].max
			else
				return 0
			end
		end
		
		
		def get_damage_types(actor)
			@damage_types
		end
		

		
		# 外れたなら0、命中したならその命中回数、回数が問題にならない成功ならnil
		def calc_hit_number(actor, target, clean_hit = false)
			border = get_hitting_border(actor, target)
			attack_number = get_attack_number(actor, target)
			
			# 対霊体処理（攻撃が非常に当たりにくい）
			if border && check_ghost_body_effect(actor, target) then
				LOGGER.puts "\t\t霊体に対しての攻撃は当たりにくい"
				border = 25
			end
			

		
			# 攻撃回数が設定されているか？
			if attack_number then
				# 命中判定
				hit_number = 0
				attack_number.times do
					hit_number += 1 if clean_hit || !(target.active?) || Roll.check_3d33(border)
				end
				LOGGER.puts "\t\t命中率#{ROLL_TABLE[border]}% : #{attack_number}回攻撃 : #{hit_number}回命中"

			else
				# 成功判定(border=nilは絶対成功)
				if border.nil? then
					hit_number = nil
					LOGGER.puts "\t\t絶対成功"

				elsif Roll.check_3d33(border) then
					hit_number = nil
					LOGGER.puts "\t\t命中率#{ROLL_TABLE[border]}% : 成功"

				else
					hit_number = 0
					LOGGER.puts "\t\t命中率#{ROLL_TABLE[border]}% : 失敗"

				end
			end
			
			return hit_number

		end
		
		def get_hitting_border(actor, target)
			@hitting_border
		end
		
		def get_attack_number(actor, target)
			@attack_number
		end
		
		def check_ghost_body_effect(actor, target)
			if target.is_ghost_body then
				unless $battle && $battle.praying? then
					return true
				end
			end
			
			return false
		end
		
		def check_clean_hit(actor, target)
			res = target.get_damage_resist(get_damage_types(actor))
			border = RES_CLEAN_HIT_BORDER_TABLE[res]
			return border && Roll.check_3d33(border)
		end

		def check_shield_guard(actor, target)
			
			if target.shield_usable? then
				if target.action.is_a?(GuardAction)
					LOGGER.puts "\t\t盾：#{target.shield.name} (ガード状態/100%)"
					return true  # ガード中は絶対成功
				else
				
					border = get_shield_border(actor, target)
					LOGGER.puts "\t\t盾:#{target.shield.name} (#{ROLL_TABLE[border]})%"
					return(Roll::Dice.new.roll(border) >= Roll::SUCCESS)
				end
			else
				return false
			end
		end
		
		def get_shield_border(actor, target)
			border = target.shield.guard_border
			border -= ((actor.tec - target.tec) / 3).to_i if actor
			return border
		end
	end
	
	class SingleAttackEffect < AttackEffect
		def target_type
			TG_ENEMY
		end
		
		def operate(actor, targets)
			super(actor, [targets.first])
		end
	end
	
	class StateDamageEffect < AttackEffect
		def init_params
			super
			@through_defense = true
			@shield_guardable = false
		end
		
		def operate(target)
			super(nil, [target])
		end
	end
	
	
	class GroupAttackEffect < AttackEffect
		def target_type
			TG_ENEMY
		end
	end
	
	class WeaponAttackEffect < AttackEffect
		attr_accessor :damage_rate
		
		def init_params
			super
			@damage_rate = 1.0
		end
		
		def target_type
			return TG_ENEMY
		end
		
		def operate(actor, targets)
			rate = success_rate(get_hitting_border(actor, targets.first))
			actor.on_use_weapon(actor.attack_number)
			
			return super(actor, [targets.first])
		end
		
		def get_message(actor, targets)
			msg = actor.weapon.get_attack_message
			return eval_message(msg, :actor => actor.name, :target => self.target_name(targets))
		end


	
		def get_attack(actor)
			atk = attack_bonus(actor, @damage_rate)
			atk += @fixed_damage if @fixed_damage
			atk *= actor.arrow.damage_rate if actor.weapon.data.is_a?(BowModel)
			return atk
		end
		
		def get_damage_types(actor)
			actor.weapon.damage_types
		end
		

		
		
		def get_hitting_border(actor, target)
			border = 50 + actor.weapon_hitting - target.avoid
			border -= 20 if target.guarding?
			border += 5 if target.action && target.action.unguarding?
			
			return border
		end
		
		def get_attack_number(actor, target)
			actor.attack_number
		end
				
		def attack_bonus(actor, rate)
			return (actor.weapon_power * rate).to_i
		end
		
		
		def use_skill(actor)
			if defined?(actor.weapon_mastery) then
				symbol = actor.weapon.data_symbol
				if actor.masterable?(actor.weapon) then
					actor.weapon_mastery[symbol] ||= 0.0
					actor.weapon_mastery[symbol] += 0.2
					LOGGER.log(UseWeaponLog, actor, 0.2)
				end
			end
		end
		
		
		
	end
	
	class RanpageEffect < WeaponAttackEffect
		def get_attack_number(actor, target)
			super * 2
		end
		def get_message(actor, targets)
			msg = actor.weapon.get_ranpage_message
			return eval_message(msg, :actor => actor.name, :target => self.target_name(targets))
		end
	end
	
	class FeintEffect < SingleAttackEffect
		def init_params
			super
			@state_data << [FeintState.new, nil]
			@messages << "%actorはフェイントをかけた！"
		end
	end


	
	class SmashEffect < WeaponAttackEffect
		def init_params
			super
			@state_data << [SmashedState.new, 60]
		end
		
		def get_attack_number(actor, target)
			1
		end
		def get_message(actor, targets)
			msg = actor.weapon.get_smash_message
			return eval_message(msg, :actor => actor.name, :target => self.target_name(targets))
		end
		
		def get_attack(actor)
			(super * (1.2 + (0.1 * actor.attack_number))).to_i
		end
	end
	
	class ChargeAttackEffect < WeaponAttackEffect
		def initialize(level, params = {})
			super(params)
			@level = level
		end
		
		def get_attack(actor)
			(super * ChargingState::DAMAGE_RATE_BONUS_TABLE[@level - 1]).to_i
		end
		
		def get_hitting_border(actor, target)
			super + ChargingState::HITTING_BONUS_TABLE[@level - 1]
		end

	end
	

	
	class MagicAttackEffect < AttackEffect
	
		attr_accessor :damage_rate
		include MagicEffect
		
		def init_params
			super
			@damage_rate = nil
		end

		def get_attack(actor)
			return @fixed_damage + magic_attack_bonus(actor, @damage_rate)
		end
		
		def get_shield_border(actor, target)
			super - 10
		end

	end
	
	class MagicSingleAttackEffect < MagicAttackEffect
		def target_type
			return TG_ENEMY
		end
		
		def operate(actor, targets)
			super(actor, [targets.first])
		end
		

	end
	
	class MagicGroupAttackEffect < MagicAttackEffect
		def target_type
			return TG_ENEMY
		end
		
		def operate(actor, targets)
			case actor
			when Member
				super(actor, targets)
			when Enemy
				super(actor, $party.members)
			end
		end

	end

	
	# HP・MP・CT・状態異常の回復
	# 回復しないパラメータにはnilを入れる
	class RecoverEffect < Effect
		attr_accessor :hp_fixed_recovery, :mp_fixed_recovery
		attr_accessor :state_data, :add_status, :target_type
		
		def init_params
			super
			@hp_fixed_recovery = nil
			@mp_fixed_recovery = nil
			@state_data = []
			@add_status = []
			init_usability(true, true, true)
		end
		

		def calc_recover_hp(actor)
			return nil unless @hp_fixed_recovery
			recovery = @hp_fixed_recovery
			return double_roll(recovery * 2, recovery, get_balance(actor))
		end
		
		
		def get_balance(actor)
			0.6
		end
		
		
		
		def operate(actor, targets = $party.members)
			
			result = [get_message(actor, targets)]
			targets.each do |target|
				msgs = []
				# HP回復
				value = calc_recover_hp(actor)
				if value && target.hp < target.hp_max then
					target.recover_hp(value)
					target.update_last_parameters
					msgs << Message.on_hp_cured(target, value)
				end
				# 状態異常治療
				@state_data.each do |cls, level|
					already = target.added_status.find{|x| x.class == cls}
					next unless already
					
					msgs << already.get_recover_message(target)
					target.added_status.delete(already)
				end
				
				# 状態異常付加
				@add_status.each do |state|
					target.add_state(state)
					
					msgs << state.get_added_message(target)
				end

				
				result << (msgs.empty? ? Message.on_no_effect(target) : msgs.join("\n"))
			end
			
			
			
			return result
		end

	end
	
	
	
	
	class MagicRecoverEffect < RecoverEffect
		attr_accessor :hp_recovery_rate, :mp_recovery_rate
		
		def init_params
			super
			@hp_recovery_rate = nil
			@mp_recovery_rate = nil
		end
		
		def set_hp_recovery(fixed, rate)
			@hp_fixed_recovery = fixed
			@hp_recovery_rate = rate
		end

		def calc_recover_hp(actor)
			return nil unless @hp_fixed_recovery && @hp_recovery_rate
			recovery = @hp_fixed_recovery + (actor.magic_power * @hp_recovery_rate).to_i
			return double_roll(recovery * 2, recovery, get_balance(actor))
		end

		def get_balance(actor)
			control = actor.magic_control - actor.magic_power
			b = 0.6 + control * 0.05
			return DarkHall.fold_in_range(b, (0.1)..(0.9))
		end
		
		
	end
	

	
	class MagicGroupRecoverEffect < RecoverEffect
		
		def init_params
			super
			@target_type = TG_NONE
		end
		
		def operate(actor, targets = $party.alive_members)
			super(actor, targets)
		end

		
		
	end

	

	

	
	
	
	class MapVisionEffect < Effect
		def init_params
			super
			@camp_usable = true
		end
		
		def operate(actor)
			message("#{actor.name}は幻視の呪文を唱えた！")
			Phase.change(DungeonMapPhase.new)
			return nil
		end
	end
	
	
end
