# ウインドゥクラス



module DarkHall
	PADDING = 4
	DOCK_MARGIN = 8
	AL_LEFT = 1
	AL_CENTER = 2
	AL_RIGHT = 3
	
	class Window
	
		SPACE = PADDING
	
		SIDE_WINDOW_WIDTH = 200
		attr_accessor :left, :top, :width, :height, :back_surface, :front_surface
		attr_accessor :silent, :has_frame, :has_caption
	
		def initialize
			@left = 0
			@top = 0
			@width ||= 100
			@height ||= 100
			@has_frame = true
			@has_caption = false
		end
		
		# 一行の高さ
		def line_height
			return REGULAR_FONT.line_skip + 6
		end
		
		def line_top(line_index)
			return(line_height * line_index)
		end
		
		def line_text_top(line_index)
			return self.line_top(line_index) + ((line_height - REGULAR_FONT.height) / 2).to_i
		end

		
		# 幅・高さの設定に応じてサーフェイスを作成する
		def make_surface(width = nil, height = nil)
			@width = width if width
			@height = height if height
			
			@back_surface = SDL::Surface.new(SDL::SRCALPHA, @width, @height, $screen)
			@back_surface.fill_rect(0, 0, @width, @height, ColorTable::BLACK)
			@back_surface.set_alpha(SDL::SRCALPHA | SDL::RLEACCEL, 200) if Config::ENABLE_ALPHA_BG
			@back_surface.display_format
			
			# グラデーション
			#step = 10
			#step.times do |i|
			#	c = i * 6
			#	@back_surface.fill_rect(0, (@height / step).to_i * i, @width, (@height / step).to_i, [c, c, c])
			#end
			
			@front_surface = SDL::Surface.new(SDL::SRCCOLORKEY, self.width, self.height, $screen)
			@front_surface.set_color_key(SDL::SRCCOLORKEY, 0)
			self.update
			@front_surface.display_format
			
			return self
		end
		
		# サーフェイス書き換え
		def update
			self.clear_surface
			return self
		end
		
		def clear_surface
			@front_surface.fill_rect(0, 0, @width - 1, @height - 1, ColorTable::BLACK)
			@front_surface.draw_rect(0, 0, @width - 1, @height - 1, ColorTable::WHITE) if @has_frame
			@front_surface.fill_rect(0, PADDING + line_top(1) - 1, @width - 1, 1, ColorTable::WHITE) if @has_caption
			@front_surface.unlock
			return self
		end
		
		def break_surface
			@front_surface = nil
			@back_surface = nil
			@cursor_surface = nil
		end
		
		def maked_surface?
			return(@front_surface && @back_surface)
		end

		# スクリーンにウインドゥを表示
		def blit
			$screen.put(@back_surface, @left, @top)
			$screen.put(@front_surface, @left, @top)
			return self
		end
		

	
		def client_width
			return(self.width - PADDING * 2)
		end
	
		def client_height
			return(self.height - PADDING * 2)
		end
		
		def client_width=(w)
			@width = w - PADDING*2
		end

		def client_height=(h)
			@height = h - PADDING*2
		end


		def client_left
			return @left + PADDING
		end
		
		def client_top
			return @top + PADDING
		end

		
		def right
			return(@left + @width)
		end
		
		def right=(value)
			@left = value - @width
		end
	
		
		def bottom
			return(@top + @height)
		end
		
		def bottom=(value)
			@top = value - @height
		end
		
		def set_height_from_line(line, font = REGULAR_FONT)
			@height = PADDING * 2 + line_height * line
		end
	
		
		def set_position(x, y)
			if x == :center then
				@left = SCREEN_WIDTH/2 - @width/2
			elsif x.nil? then
				# 何もしない
			else
				@left = x
			end
			
			if y == :center then
				@top = SCREEN_HEIGHT/2 - @height/2
			elsif y.nil? then
				# 何もしない
			else
				@top = y
			end
			
			return self
		end
		
		# ウインドゥの側に移動
		def dock_beside(window, side, margin = DOCK_MARGIN)
			case side
			when :top
				self.set_position(window.left, window.top - self.height - margin)
			when :bottom, :under
				self.set_position(window.left, window.bottom + margin)
			when :left
				self.set_position(window.left - self.width - margin, window.top)
			when :right
				self.set_position(window.right + margin, window.top)
			end
			
			return self
		end
		
		def dock_under(window)
			self.set_position(window.left + 24, window.top + 24)
			
			return self
		end


		def centering(party_window_ignore = false)
			@left = SCREEN_WIDTH/2 - @width/2
			area_height = (party_window_ignore ? SCREEN_HEIGHT : (SCREEN_HEIGHT - PARTY_WINDOW.height))
			@top = area_height/2 - @height/2
			return self
		end
		
		def centering_on_window(window)
			@left = window.left + (window.width - @width)/2
			@top = window.top + (window.height - @height)/2
			return self
		end


		def show
			if $windows.include?(self) then
				$windows.delete(self)
				$windows << self
			else
				$windows << self
			end
			
			return self
		end
		
		
		def hide
			$windows.delete(self)
			return self
		end



		def shown?
			return($windows.include?(self))
		end
		
		def warp
		end
		
		
		TAG_PATTERN = /^\$\[(.+?)\]/
		TAGS = {}
		TAGS[:goodcolor] = 'goodcolor'
		TAGS[:badcolor] = 'badcolor'
		TAGS[:disablecolor] = 'disablecolor'
		TAGS[:lightdisablecolor] = 'lightdisablecolor'
		TAGS[:solid] = 'solid'
		
		CHAR_LENGTH_TABLE = []
		CHAR_LENGTH_TABLE[1] = 1
		CHAR_LENGTH_TABLE[2] = 2
		CHAR_LENGTH_TABLE[3] = 2
		def draw_text(align, text, x, y, width = self.client_width, color = ColorTable::WHITE, font = REGULAR_FONT)
			text = text.to_s
			char_width = font.text_size(' ')[0]
			
			if text =~ TAG_PATTERN then
				case $1.downcase
				when TAGS[:goodcolor]
					color = ColorTable::GOOD
				when TAGS[:badcolor]
					color = ColorTable::BAD
				when TAGS[:disablecolor]
					color = ColorTable::DISABLE
				when TAGS[:solid]
					solid_draw = true

				end
				text = $'
			end
			
			case align
			when AL_LEFT
				left = x + PADDING

			when AL_CENTER
				text_width = 0
				text.each_char do |c|
					text_width += (char_width * CHAR_LENGTH_TABLE[c.length])
				end

				left = x + PADDING + (width - text_width) / 2
			when AL_RIGHT
				text_width = 0
				text.each_char do |c|
					text_width += (char_width * CHAR_LENGTH_TABLE[c.length])
				end

				left = x + PADDING + width - text_width
			end

			text.each_char do |c|
				font.draw_blended_utf8(@front_surface, c, left, y + PADDING, color[0], color[1], color[2])
				left += (char_width * CHAR_LENGTH_TABLE[c.length])
			end
			
		end
	
		def draw_text_left(text, x, y, color = ColorTable::WHITE, font = REGULAR_FONT)
			self.draw_text(AL_LEFT, text, x, y, self.client_width, color, font)
		end
	
		def draw_text_right(text, x, y, width = self.client_width, color = ColorTable::WHITE, font = REGULAR_FONT)
			self.draw_text(AL_RIGHT, text, x, y, width, color, font)
		end
		
		def draw_text_center(text, x, y, width = self.client_width, color = ColorTable::WHITE, font = REGULAR_FONT)
			self.draw_text(AL_CENTER, text, x, y, width, color, font)
		end
		
		def on_enter
			self.on_enter_key
		end
		
		def on_enter_key
			$phase.on_enter_key
		end
		
		def on_cancel
			$phase.on_cancel
		end
		
		def on_left_key
			if defined?($phase.on_left_key) then
				$phase.on_left_key
			else
				$phase.on_key_press(SDL::Key::LEFT)
			end
		end
		
		def on_right_key
			if defined?($phase.on_right_key) then
				$phase.on_right_key
			else
				$phase.on_key_press(SDL::Key::RIGHT)
			end
		end
		
		def on_key_press(keycode)
			$phase.on_key_press(keycode)

		end
		
		def on_shift_down
			$phase.on_shift_down
		end
		
		def on_shift_up
			$phase.on_shift_up

		end

		def on_ctrl_down
			$phase.on_ctrl_down
		end
		
		def on_ctrl_up
			$phase.on_ctrl_up
		end

		def on_change
		end
		
	end
	
	module TemporaryWindow
		def on_select(id)
			throw(:exit_operation_loop, id)
		end
		
		def on_cancel
			throw(:exit_operation_loop, nil)
		end
	end
	
	class DungeonWindow < Window
		HORIZONAL_LENGTH_DATA = [140, 105, 70, 40, 20]
		# HORIZONAL_LENGTH_DATA = [140, 105, 70, 40, 20]
		VERTICAL_LENGTH_DATA = HORIZONAL_LENGTH_DATA
		WINDOW_WIDTH = 250
		WINDOW_HEIGHT = 250
		WALL_COLORS = []
		DOOR_COLORS = []
		

	
		def initialize
			@width = WINDOW_WIDTH + 1 * 2
			@height = WINDOW_HEIGHT + 1 * 2
		end
		
		def DungeonWindow.get_points(range, position)
			return POINTS[range][position]
		end
		
		POINTS = []
		(0..(VIEWABLE_FRONT_RANGE)).each do |range|
			POINTS[range] = {}
			((VIEWABLE_SIDE_RANGE * -1)..VIEWABLE_SIDE_RANGE).each do |pos|
				position_x = pos * (HORIZONAL_LENGTH_DATA[range] * 2 + 1)
				px1 = (WINDOW_WIDTH / 2) - 1 - HORIZONAL_LENGTH_DATA[range] + position_x
				py1 = (WINDOW_HEIGHT / 2) - 1 - VERTICAL_LENGTH_DATA[range]
				px2 = (WINDOW_WIDTH / 2) + HORIZONAL_LENGTH_DATA[range] + position_x
				py2 = (WINDOW_HEIGHT / 2) + VERTICAL_LENGTH_DATA[range]
				
				points = []
				points[0] = [px1, py1]
				points[1] = [px2, py1]
				points[2] = [px1, py2]
				points[3] = [px2, py2]
				
				POINTS[range][pos] = points

			end
			
		end
		
		
		def make_surface
		
			(VIEWABLE_FRONT_RANGE + 1).times do |i|
				c = 100 - i * 16
				WALL_COLORS[i] = ($game_config.dungeon_mapping == GameConfig::DM_COLORFUL ? [c, c, c] : ColorTable::BLACK)
				DOOR_COLORS[i] = ($game_config.dungeon_mapping == GameConfig::DM_COLORFUL ? Color::DOOR : ColorTable::BLACK)
			end

			unless @front_wall_surfaces then
				@front_wall_surfaces = []
				@front_door_surfaces = []
				(0..(VIEWABLE_FRONT_RANGE)).each do |range|
					points = DungeonWindow.get_points(range, 0)
					width = points[1][0] - points[0][0] + 1
					height = points[2][1] - points[0][1] + 1
					
					# 壁
					surface = SDL::Surface.new(0, width, height, $screen)
					surface.draw_rect(0, 0, width - 1, height - 1, ColorTable::WHITE)
					surface.fill_rect(1, 1, width - 1 - 1, height - 1 - 1, WALL_COLORS[range])
					surface.unlock
					surface.display_format
					@front_wall_surfaces[range] = surface
					
					# ドア
					surface = surface.display_format
					door_w = (width * 0.6).to_i
					door_h = (height * 0.8).to_i
					surface.draw_rect(width / 2 - door_w / 2, height - door_h, door_w, door_h, ColorTable::WHITE)
					surface.fill_rect(width / 2 - door_w / 2 + 1, height - door_h + 1, door_w - 1, door_h - 1, DOOR_COLORS[range])
					surface.draw_rect(0, 0, width - 1, height - 1, ColorTable::WHITE)
					surface.unlock
					surface.display_format
					@front_door_surfaces[range] = surface
				end
		
				@left_wall_surfaces = []
				@right_wall_surfaces = []
				@left_door_surfaces = []
				@right_door_surfaces = []
				(0...(VIEWABLE_FRONT_RANGE)).each do |range|
					@left_wall_surfaces[range] = {}
					@right_wall_surfaces[range] = {}
					@left_door_surfaces[range] = {}
					@right_door_surfaces[range] = {}
					(1..(VIEWABLE_SIDE_RANGE)).each do |pos|
						front_points = POINTS[range][pos]
						left_points = POINTS[range + 1][pos - 1]
						width = front_points[0][0] - left_points[1][0] + 1
						height = front_points[2][1] - front_points[0][1]
						next if width <= -1
						
						surface = SDL::Surface.new(0, width, height, $screen)
		
						x1 = x2 = width - 1
						x3 = x4 = 0
						y1 = 0
						y2 = height - 1
						y3 = left_points[0][1] - front_points[0][1]
						y4 = left_points[2][1] - front_points[0][1]

						surface.set_color_key(SDL::SRCCOLORKEY | SDL::RLEACCEL, ColorTable::AQUA)
						surface.fill_rect(0, 0, width - 1, height - 1, ColorTable::AQUA)
						surface.draw_line(x1, y1, x3, y3, ColorTable::WHITE)
						surface.draw_line(x2, y2, x4, y4, ColorTable::WHITE)
						surface.draw_line(x1, y1, x2, y2, ColorTable::WHITE)
						surface.draw_line(x3, y3, x4, y4, ColorTable::WHITE)
						door_surface = surface.display_format
						door_w = (width * 0.6).to_i
						door_left = width / 2 - door_w / 2
						door_right = door_left + door_w - 1
						
						white_pixel = surface.map_rgb(*ColorTable::WHITE)
						door_p1x = door_p1y = door_p2x = door_p2y = 0
						
						(1..(width - 1)).each do |x|
							y = 0
							
							until surface.get_pixel(x, y) == white_pixel do
								y += 1
							end
							
							y += 1
							top = y
		
							until surface.get_pixel(x, y) == white_pixel do
								y += 1
							end
							
							height = y - top
							door_h = (height * 0.8).to_i
							door_y = top + height - door_h
						
							surface.fill_rect(x, top, 1, height, WALL_COLORS[range])
							door_surface.fill_rect(x, top, 1, height - 1, WALL_COLORS[range])
							
							if x == door_left then
								door_surface.fill_rect(x, door_y, 1, door_h, ColorTable::WHITE)
								door_p1x = x; door_p1y = door_y
							elsif x == door_right then
								door_surface.fill_rect(x, door_y, 1, door_h, ColorTable::WHITE)
								door_p2x = x; door_p2y = door_y
							elsif (door_left..door_right).include?(x) then
								door_surface.fill_rect(x, door_y, 1, door_h, DOOR_COLORS[range])
								door_surface.fill_rect(x, door_y, 1, 1, ColorTable::WHITE)
							end
						end
						
						surface.unlock
						surface.display_format
						@left_wall_surfaces[range][pos] = surface

						door_surface.unlock
						door_surface.display_format
						@left_door_surfaces[range][pos] = door_surface

					end
					
					((VIEWABLE_SIDE_RANGE * -1)..-1).each do |pos|
						front_points = POINTS[range][pos]
						right_points = POINTS[range + 1][pos + 1]
						width = right_points[0][0] - front_points[1][0] + 1
						height = front_points[2][1] - front_points[0][1]
						next if width <= -1
						
						surface = SDL::Surface.new(0, width, height, $screen)
		
						x1 = x2 = 0
						x3 = x4 = width - 1
						y1 = height - 1
						y2 = 0
						y4 = right_points[0][1] - front_points[0][1]
						y3 = right_points[2][1] - front_points[0][1]
						
						surface.set_color_key(SDL::SRCCOLORKEY | SDL::RLEACCEL, ColorTable::AQUA)
						surface.fill_rect(0, 0, width - 1, height - 1, ColorTable::AQUA)
						surface.draw_line(x1, y1, x3, y3, ColorTable::WHITE)
						surface.draw_line(x2, y2, x4, y4, ColorTable::WHITE)
						surface.draw_line(x1, y1, x2, y2, ColorTable::WHITE)
						surface.draw_line(x3, y3, x4, y4, ColorTable::WHITE)
						door_surface = surface.display_format
						door_w = (width * 0.6).to_i
						door_left = width / 2 - door_w / 2
						door_right = door_left + door_w - 1
						
						white_pixel = surface.map_rgb(*ColorTable::WHITE)
						door_p1x = door_p1y = door_p2x = door_p2y = 0

						(1..(width - 1)).each do |x|
							y = 0
							
							until surface.get_pixel(x, y) == white_pixel do
								y += 1
							end
							
							y += 1
							top = y
		
							until surface.get_pixel(x, y) == white_pixel do
								y += 1
							end
							
							height = y - top
							door_h = (height * 0.8).to_i
							door_y = top + height - door_h
						
							surface.fill_rect(x, top, 1, height, WALL_COLORS[range])
							door_surface.fill_rect(x, top, 1, height, WALL_COLORS[range])
							
							if x == door_left then
								door_surface.fill_rect(x, door_y, 1, door_h, ColorTable::WHITE)
								door_p1x = x; door_p1y = door_y
							elsif x == door_right then
								door_surface.fill_rect(x, door_y, 1, door_h, ColorTable::WHITE)
								door_p2x = x; door_p2y = door_y
							elsif (door_left..door_right).include?(x) then
								door_surface.fill_rect(x, door_y, 1, door_h, DOOR_COLORS[range])
								door_surface.fill_rect(x, door_y, 1, 1, ColorTable::WHITE)
							end
						
						end
						
						surface.unlock
						surface.display_format
						@right_wall_surfaces[range][pos] = surface

						door_surface.unlock
						door_surface.display_format
						@right_door_surfaces[range][pos] = door_surface

					end
	
				end
			end
			super
			
			
			
			#save_wall_surfaces
	
			return self
		end
		
		def save_wall_surfaces
			
			@left_wall_surfaces.each_index do |range|
				@left_wall_surfaces[range].each_pair do |pos, surface|
					surface.save_bmp("surface/leftwall_#{range}_#{pos}.bmp") 
				end
			end
			
			@right_wall_surfaces.each_index do |range|
				@right_wall_surfaces[range].each_pair do |pos, surface|
					surface.save_bmp("surface/rightwall_#{range}_#{pos.abs}.bmp") 
				end
			end

		end
		
	
		def update
			super
			floor = $party.floor
			area_data = floor.get_viewable_area_data($party)
			side_range = VIEWABLE_SIDE_RANGE
			
			range = VIEWABLE_FRONT_RANGE
			while range >= 0 do
				# 初回だけスキップ
				unless range == VIEWABLE_FRONT_RANGE then
					((side_range * -1)..-1).each do |pos|
						case area_data[range * 2][pos * 2 + 1]
						when Dungeon::Area::WALL
							draw_right_wall(range, pos)
						when Dungeon::Area::DOOR, Dungeon::Area::EXIT_DOOR
							draw_right_door(range, pos)
						end
					end
					
					pos = side_range
					while pos >= 1 do
						case area_data[range * 2][pos * 2 - 1]
						when Dungeon::Area::WALL
							draw_left_wall(range, pos)
						when Dungeon::Area::DOOR, Dungeon::Area::EXIT_DOOR
							draw_left_door(range, pos)
						end
						pos -= 1
					end
				end
				

				((side_range * -1)..(side_range)).each do |pos|
					break if range == 0
					case area_data[range * 2 - 1][pos * 2]
					when Dungeon::Area::WALL
						draw_front_wall(range, pos)
					when Dungeon::Area::DOOR, Dungeon::Area::EXIT_DOOR
						draw_front_door(range, pos)
					end
				end

				
				range -= 1
				
			end
			
			@front_surface.draw_rect(0, 0, @width - 1, @height - 1, ColorTable::WHITE)
			@front_surface.unlock
			
			dir = %W(N E W S)[$party.direction]
			room = ($party.room_id ? "room#{$party.room_id}" : 'way')
			self.draw_text_left("#{$party.x}, #{$party.y}, #{dir}, #{room}", 0, 0) if DEBUG
			
			#dup = @front_surface.display_format
			# SDL.transform(src,dst,angle,xscale,yscale,px,py,qx,qy,flags)
			#SDL.transform(dup, @front_surface, 0, 2.0, 2.0, @front_surface.w / 2, @front_surface.h / 2, @front_surface.w / 2, @front_surface.h / 2, SDL::TRANSFORM_AA)
			
			
			
			return self	
		end
		
		
		
		



		
		def draw_front_wall(range, position)
			points = DungeonWindow.get_points(range, position)
			x, y = points[0]
			@front_surface.put(@front_wall_surfaces[range], x, y)
		end
		
		def draw_front_door(range, position)
			points = DungeonWindow.get_points(range, position)
			x, y = points[0]
			@front_surface.put(@front_door_surfaces[range], x, y)
		end
		
		def draw_left_wall(range, position)
		

			return if range >= VIEWABLE_FRONT_RANGE
			front_points = POINTS[range][position]
			left_points = POINTS[range + 1][position - 1]

			surface = @left_wall_surfaces[range][position]
			return unless surface
			
			x = left_points[1][0]
			y = front_points[0][1]
			@front_surface.put(surface, x, y)

		end

		def draw_left_door(range, position)
		

			return if range >= VIEWABLE_FRONT_RANGE
			front_points = POINTS[range][position]
			left_points = POINTS[range + 1][position - 1]

			surface = @left_door_surfaces[range][position]
			return unless surface
			
			x = left_points[1][0]
			y = front_points[0][1]
			@front_surface.put(surface, x, y)

		end

		
		def draw_right_wall(range, position)
		
			return if range >= VIEWABLE_FRONT_RANGE
			front_points = POINTS[range][position]
			right_points = POINTS[range + 1][position + 1]
			
			surface = @right_wall_surfaces[range][position]
			return unless surface
			
			x = front_points[1][0]
			y = front_points[1][1]
			@front_surface.put(surface, x, y)

		end
		
		def draw_right_door(range, position)
		

			return if range >= VIEWABLE_FRONT_RANGE
			front_points = POINTS[range][position]
			right_points = POINTS[range + 1][position + 1]

			surface = @right_door_surfaces[range][position]
			return unless surface
			
			x = front_points[1][0]
			y = front_points[1][1]
			@front_surface.put(surface, x, y)

		end
		
		def draw_down_stair(range, position)
			return
			p1 = POINTS[range][position]
			p2 = POINTS[range + 1][position]
			
			center_x = (p1[0][0] + p1[1][0]) / 2
			center_y = (p2[2][1] + p1[2][1]) / 2

			
			@front_surface.fill_rect(x, y, width, height, ColorTable::WHITE)
		end
		
		
	end
	
	DUNGEON_WINDOW = DungeonWindow.new
	
	class DungeonMapWindow < Window
	
		attr_accessor :origin_x, :origin_y, :cursor_x, :cursor_y, :begin_x, :begin_y
		attr_accessor :range_cursor_surface, :cursor_surface, :all_found, :wall_mode
		
		VIEWABLE_WIDTH = 30
		VIEWABLE_HEIGHT = 24
		MODE_NORMAL = 0
		MODE_WALL_EDIT = 1
	
		def initialize
			@width = VIEWABLE_WIDTH * (GRID + 1) + 1 * 2
			@height = VIEWABLE_HEIGHT * (GRID + 1) + 1 * 2
			@origin_x = 0
			@origin_y = 0
			@cursor_x = 0
			@cursor_y = 0
			@all_found = false
			@wall_mode = false
		end
		
		GRID = 10
		
		def reset_origin
			@origin_x = $party.x - VIEWABLE_WIDTH / 2
			@origin_y = $party.y - VIEWABLE_HEIGHT / 2
			@cursor_x = $party.x
			@cursor_y = $party.y
			@begin_x = nil
			@begin_y = nil
			return self
		end
		
		def up_cursor
			if ranging? && @cursor_y == @begin_y then
				return
			elsif @cursor_y >= 1 then
				@cursor_y -= 1
				@origin_y -= 1 if (@cursor_y - @origin_y) <= 0
			end

			
		end
		
		def down_cursor
			if @cursor_y < FLOORS[$party.floor_id].height then
				@cursor_y += 1
				@origin_y += 1 if (@cursor_y - @origin_y) >= VIEWABLE_HEIGHT
			end
		end
		
		def right_cursor
			if @cursor_x < FLOORS[$party.floor_id].width then
				@cursor_x += 1
				@origin_x += 1 if (@cursor_x - @origin_x) >= VIEWABLE_WIDTH
			end
		end
		
		def left_cursor
			if ranging? && @cursor_x == @begin_x then
				return
			elsif @cursor_x >= 1 then
				@cursor_x -= 1
				@origin_x -= 1 if (@cursor_x - @origin_x) <= 0
			end

		end
		
		def set_begin
			@begin_x = @cursor_x
			@begin_y = @cursor_y
			return self
		end
		
		def clear_begin
			@begin_x = @begin_y = nil
			return self
		end
		
		def ranging?
			return(@begin_x && @begin_y)
		end
		
		def found_area?(x, y)
			return(@all_found || $found_area[$party.floor_id][x][y] == 1)
		end
		
		def make_surface
			super
			
			floor = $party.floor
			@map_surface = SDL::Surface.new(SDL::SRCCOLORKEY, floor.width * GRID, floor.height * GRID, $screen)
			surface = @map_surface
			
			(0...(floor.width)).each do |x|
				(0...(floor.height)).each do |y|
					surface.draw_rect(draw_x(x), draw_y(y), GRID + 1, GRID + 1, ColorTable::MAP_GRID)
				end
			end
			
			(0...(floor.width)).each do |x|
				(0...(floor.height)).each do |y|
					
				
					# 床
					if found_area?(x, y) then
						case floor.flooring(x, y)
						when Dungeon::Area::ROOM, Dungeon::Area::ROUTE then
							color = (floor.flooring(x, y) == Dungeon::Area::ROOM ? ColorTable::MAP_ROOM : ColorTable::MAP)
							surface.fill_rect(draw_x(x), draw_y(y), GRID + 2, GRID + 2,color)
							
							length = GRID + 2
							door_length = 2
							door_margin = 3
	
							# 北の壁
							area = floor.wall(DIR_N, x, y)
							case area
							when Area::WALL, Area::DOOR, Area::EXIT_DOOR
								surface.fill_rect(draw_x(x), draw_y(y), length, 1, ColorTable::WHITE)
								case area
								when Area::DOOR, Area::EXIT_DOOR
									surface.fill_rect(draw_x(x) + door_margin, draw_y(y) + 1, 1, door_length, ColorTable::WHITE)
									surface.fill_rect(draw_x(x + 1) - door_margin, draw_y(y) + 1, 1, door_length, ColorTable::WHITE)
								end
							end
						
							# 東の壁
							area = floor.wall(DIR_E, x, y)
							case area
							when Area::WALL, Area::DOOR, Area::EXIT_DOOR
								surface.fill_rect(draw_x(x + 1), draw_y(y), 1, length, ColorTable::WHITE)
								case area
								when Area::DOOR, Area::EXIT_DOOR
									surface.fill_rect(draw_x(x + 1) - door_length, draw_y(y) + door_margin, door_length, 1, ColorTable::WHITE)
									surface.fill_rect(draw_x(x + 1) - door_length, draw_y(y + 1) - door_margin, door_length, 1, ColorTable::WHITE)
								end
							end
		
							# 南の壁
							area = floor.wall(DIR_S, x, y)
							case area
							when Area::WALL, Area::DOOR, Area::EXIT_DOOR
								surface.fill_rect(draw_x(x), draw_y(y + 1), length, 1, ColorTable::WHITE)
								case area
								when Area::DOOR, Area::EXIT_DOOR
									surface.fill_rect(draw_x(x) + door_margin, draw_y(y + 1) - door_length, 1, door_length, ColorTable::WHITE)
									surface.fill_rect(draw_x(x + 1) - door_margin, draw_y(y + 1) - door_length, 1, door_length, ColorTable::WHITE)
								end
							end
		
							# 西の壁
							area = floor.wall(DIR_W, x, y)
							case area
							when Area::WALL, Area::DOOR, Area::EXIT_DOOR
								surface.fill_rect(draw_x(x), draw_y(y), 1, length, ColorTable::WHITE)
								case area
								when Area::DOOR, Area::EXIT_DOOR
									surface.fill_rect(draw_x(x) + 1, draw_y(y) + door_margin, door_length, 1, ColorTable::WHITE)
									surface.fill_rect(draw_x(x) + 1, draw_y(y + 1) - door_margin, door_length, 1, ColorTable::WHITE)
								end
							end
						end
					end
						
				end
			end
			surface.fill_rect(draw_x($party.x) + 2, draw_y($party.y) + 2, GRID - 1*2, GRID - 1*2, [255, 255, 0])
			($dungeon_parties - [$party]).each do |party|
				surface.fill_rect(draw_x(party.x) + 2, draw_y(party.y) + 2, GRID - 1*2, GRID - 1*2, [60, 255, 60])
			end
						
			surface.unlock
			@map_surface = surface.display_format
			
			@cursor_surface = SDL::Surface.new(SDL::SRCALPHA, (GRID + 1) + 1,(GRID + 1) + 1, $screen)
			@cursor_surface.fill_rect(0, 0, @cursor_surface.w - 1, @cursor_surface.h, ColorTable::SELECT)
			@cursor_surface.set_alpha(SDL::SRCALPHA, $cursor_alpha)
			@cursor_surface = @cursor_surface.display_format


			@range_cursor_surface = SDL::Surface.new(SDL::SRCALPHA, FLOORS[$party.floor_id].width * (GRID + 1) + 1, FLOORS[$party.floor_id].height * (GRID + 1) + 1, $screen)
			@range_cursor_surface.fill_rect(0, 0, @range_cursor_surface.w - 1, @range_cursor_surface.h, ColorTable::RED)
			@range_cursor_surface.set_alpha(SDL::SRCALPHA, $cursor_alpha)
			@range_cursor_surface = @range_cursor_surface.display_format
			return self
		end
		
		# 部屋を設定する
		def set_room
			FLOORS[$party.floor_id].set_room(@begin_x, @begin_y, (@cursor_x - @begin_x) + 1, (@cursor_y - @begin_y) + 1)
			static_make_surface

		end
		
		# 表示位置やカーソル位置を動かさずにmake_surface
		def static_make_surface
			ox, oy, cx, cy = @origin_x, @origin_y, @cursor_x, @cursor_y
			make_surface
			@origin_x, @origin_y, @cursor_x, @cursor_y = ox, oy, cx, cy
			update
		end
		
		def update
			super
			if @map_surface then
				SDL.blit_surface(@map_surface, @origin_x * (GRID + 1), @origin_y * (GRID + 1), @width, @height, @front_surface, 1, 1)
				region = FLOORS[$party.floor_id].region_data[@cursor_x][@cursor_y].to_s
				room_id = FLOORS[$party.floor_id].room_data[@cursor_x][@cursor_y]
				draw_text_left("#{@cursor_x}, #{@cursor_y}, #{region}, #{room_id}", 0, 0) if DEBUG
			end
			
			case $phase
			when DungeonMapEditPhase
				texts = []
				if ranging? then
					texts << "R:通路を置く"
					texts << "Ctrl+R:部屋を置く"
					texts << "W:範囲内を壁で埋める"
				elsif @wall_mode then
					texts << "上下左右の方向キーで置く方向を選択"
				else
					texts << "Shift+方向キー:範囲選択"
					texts << "D:ドアを置く"
					texts << "W:壁を置く"

				end
				texts.each_index do |i|
					draw_text_right(texts[i], 0, line_text_top(10 - i), @width - 4)
				end
			end
			
			return self
		end
		
		def blit
			super
			if ranging? then
				x = @left + 1 + (@begin_x - @origin_x) * (GRID + 1)
				y = @top + 1 + (@begin_y - @origin_y) * (GRID + 1)
				width = cursor_length(@cursor_x - @begin_x + 1)
				height = cursor_length(@cursor_y - @begin_y + 1)
				SDL.blit_surface(@range_cursor_surface, 0, 0, width, height, $screen, x, y)
			elsif @wall_mode then
				SDL.blit_surface(@cursor_surface, 0, 0, 0, 0, $screen, @left + 1 + (@cursor_x - @origin_x) * (GRID + 1), @top + 1 + (@cursor_y - @origin_y) * (GRID + 1))
			else
				SDL.blit_surface(@cursor_surface, 0, 0, 0, 0, $screen, @left + 1 + (@cursor_x - @origin_x) * (GRID + 1), @top + 1 + (@cursor_y - @origin_y) * (GRID + 1))
			end
		end
		
		private
		def cursor_length(length)
			return(length * (GRID + 1) + 1).abs
		end
		
		def draw_x(x)
			(GRID + 1) * x
		end
		
		def draw_y(y)
			(GRID + 1) * y
		end
	end
	
	class EnemyGraphicWindow < Window
		GROUP_MARGIN = 24
		attr_accessor :graphics
	
		def initialize
			super
			@width = SCREEN_WIDTH - 16 * 2
			@graphics = []
			@surface_positions = []
		end
		
		
		
		def clear_surface
			@front_surface.fill_rect(1, 1, @width - 1 - 2, @height - 1 - 2, Color::BLACK)
			return self
		end
		
		def update
			super
			
			set_surfaces
			@graphics.each_index do |i|
				next unless @graphics[i]
				surface = @graphics[i].surface
			
				case $game_config.enemy_display
				when GameConfig::ED_ALL
					$troop.groups[i].each do |enemy|
						if enemy.alive? then
							@front_surface.put(surface, enemy.left, enemy.top) 
						end
					end
				when GameConfig::ED_ONE
					group = $troop.groups[i]
					@front_surface.put(surface, group.first.left, group.first.top)

				end
			end
			
			return self
		end
		
		
		
		def blit
			$screen.put(@before_surface, @left, @top) if @before_surface
			$screen.put(@front_surface, @left, @top)
			return self
		end
		

		
		
		def set_surfaces
			@graphics.clear
			
			$troop.groups.each do |group|
				if group.first && group.first.graphic then
					@graphics << group.first.graphic
				else
					@graphics << nil
				end
			end

			
			return self
		end
		
		def set_surface_positions
		
			total_width = 0
			@graphics.each_index do |i|
			
				case $game_config.enemy_display
				when GameConfig::ED_ALL
					$troop.groups[i].size.times do
						total_width += @graphics[i].surface.w
						total_width += @graphics[i].margin
					end
					total_width += GROUP_MARGIN
				when GameConfig::ED_ONE
					total_width += @graphics[i].surface.w
					total_width += GROUP_MARGIN
				end
			end
			total_width -= GROUP_MARGIN
			
			cx = @width / 2 - total_width / 2
			@graphics.each_index do |i|
				case $game_config.enemy_display
				when GameConfig::ED_ALL
					$troop.groups[i].each do |enemy|
						enemy.left = cx
						enemy.top = @height - @graphics[i].surface.h - @graphics[i].height
						cx += @graphics[i].surface.w
						cx += @graphics[i].margin
					end
					cx += GROUP_MARGIN
				when GameConfig::ED_ONE
					$troop.groups[i].each do |enemy|
						enemy.left = cx
						enemy.top = @height - @graphics[i].surface.h - @graphics[i].height
					end
					cx += @graphics[i].surface.w
					cx += GROUP_MARGIN
				end
			end
			
			return self
		end
		
		

	end
	ENEMY_GRAPHIC_WINDOW = EnemyGraphicWindow.new.make_surface

=begin
	class LongTextWindow < Window
		attr_accessor :text
		def initialize
			super
			@text = text
			@length = 40
			@line = 12
		end
		

		def make_surface(length = nil, line = nil)
			w, h = BDF_FONT.text_size("全")
			@length = length if length
			@line = line if line
			width = @length * w
			height = @line * h
			super(width, height)
		end

		
		def update(text = nil)
			super()
			@text = text if text
			return self unless @text
			
			lines = @text.split($/)
			cy = 0
			lines.each do |line|
				line.each_char
			end
			
			return self
		end
	end
=end
	
	class TextWindow < Window
		attr_accessor :text
		attr_accessor :align, :with_caption
		attr_accessor :color, :font
		
		DESCRIPTION_WIDTH = REGULAR_FONT.text_size("＊" * 25)[0]
		

		
		def initialize(line = 1, text = "")
			self.set_height_from_line(line)
			super()
			@text = text
			@align = AL_LEFT
			@font = REGULAR_FONT
			@color = ColorTable::WHITE
		end
		
		def blit
			super if @text
		end


		def make_surface(width = nil, line = 1)
			self.set_height_from_line(line)
			super(width, nil)
		end
		
		def set_width_from_text(text, font = REGULAR_FONT)
			@width = font.text_size(text)[0] + PADDING*2
			return self
		end
		
		def update(text = nil)
			super()
			@text = text if text
			return self unless @text
			lines = @text.split("\n")
			
			lines.each_index do |i|
				line = lines[i]
				if @has_caption && i == 0 then
					self.draw_text_center(line, 0, line_text_top(0), self.client_width, @color, REGULAR_FONT)
				else
					case @align
					when AL_CENTER
						self.draw_text_center(line, 0, line_text_top(i), self.client_width, @color, REGULAR_FONT)
					when AL_RIGHT
						self.draw_text_right(line, 0, line_text_top(i), self.client_width, @color, REGULAR_FONT)
					else
						self.draw_text_left(line, 0, line_text_top(i), @color, REGULAR_FONT)
					end
				end
			end
			
			return self
		end
		
	end
	DESCRIPTION_WINDOW = TextWindow.new(1)
	DESCRIPTION_WINDOW.client_width = CHAR_WIDTH * 60
	DESCRIPTION_WINDOW.make_surface
	DESCRIPTION_WINDOW.set_position(:center, 16)
	INFORMATION_WINDOW = TextWindow.new.make_surface(200, 1)
	
	SEAL_WINDOW = TextWindow.new
	SEAL_WINDOW.align = AL_CENTER
	SEAL_WINDOW.make_surface(200, 1)
	
	
	CAPTION_WINDOW = TextWindow.new.make_surface(120, 1)
	CAPTION_WINDOW.align = AL_CENTER
	NAME_WINDOW = TextWindow.new.make_surface(100, 1)
	
	class CaptionWindow < TextWindow
		def initialize
			super
			@align = AL_CENTER
		end
	end
	
	class LongTextWindow < TextWindow
	end
	
	class WarningWindow < TextWindow
		def reset(width = 240)
			@text = nil

			make_surface(width) unless @width == width
			dock_beside(DarkHall.current_window, :bottom)
			return self
		end
	
		def update(selecting_item = nil)
			if selecting_item then
				@text = selecting_item.warning
			else
				@text = nil
			end
			super()
		end
		
		def blit
			super if @text
		end
	end
	WARNING_WINDOW = WarningWindow.new.make_surface(240)
	
	class MessageWindow < TextWindow
		TOP_Y = 16
		CENTER_Y = 120
		BOTTOM_Y = 200
	
		def initialize
			super
			@align = AL_CENTER
		end
		
	
		def update(text = nil)
			self.clear_surface
			@text = text if text
			lines = @text.split("\n")
			cy = (self.client_height / 2) - (REGULAR_FONT.line_skip * lines.size / 2)
			
			lines.each do |line|
				case @align
				when AL_CENTER
					self.draw_text_center(line, 0, cy, self.width, @color, REGULAR_FONT)
				when AL_RIGHT
					self.draw_text_right(line, 0, cy, self.width, @color, REGULAR_FONT)
				else
					self.draw_text_left(line, 0, cy, @color, REGULAR_FONT)
				end
				cy += REGULAR_FONT.line_skip
			end
			
			return self
		end
		
		
		
		def on_enter_key
			throw(:exit_operation_loop)
		end
		
		def on_cancel
			throw(:exit_operation_loop)
		end
		
		def on_key_press(keycode)
		end
		
		def on_left_key
		end
		
		def on_right_key
		end
	end
	MESSAGE_WINDOW = MessageWindow.new.make_surface(SCREEN_WIDTH - 16*2, 6)
	
	class SequencialMessageWindow < MessageWindow
		attr_accessor :proc
		def on_enter_key
			if @proc then
				@proc.call
			else
				throw(:exit_operation_loop)
			end
		end
	end
	SEQUENCIAL_MESSAGE_WINDOW = SequencialMessageWindow.new.make_surface(SCREEN_WIDTH - 16*2, 6)


	class DoubleTextWindow < Window
		attr_accessor :texts
		attr_accessor :color
		
		
		
		def initialize(line = 1, texts = [[]])
			self.set_height_from_line(line)
			super()
			@texts = texts
			@font = REGULAR_FONT
			@color = ColorTable::WHITE
		end

		def make_surface(width = nil, line = nil)
			self.set_height_from_line(line) if line
			super(width, nil)
		end


		def update(texts = nil)
			@texts = texts if texts
			super()
			@texts.each_index do |index|
				left_text, right_text = @texts[index]
				if @has_caption && index == 0 then
					self.draw_text_center(left_text, 0, line_text_top(index) - 2, self.client_width, @color, @font)
				else
					self.draw_text_left(left_text, 0, line_text_top(index), @color, @font)
					self.draw_text_right(right_text, 0, line_text_top(index), self.client_width, @color, @font)
				end
			end
	
			return self
		end
		
	end
	DATA_WINDOW = DoubleTextWindow.new
	HELP_WINDOW = DoubleTextWindow.new
	
	class TemporaryDoubleTextWindow < DoubleTextWindow
		def on_enter_key
			throw(:exit_operation_loop)
		end
		
		def on_cancel
			self.on_enter_key
		end
	end

	DEBUG_DATA_WINDOW = TemporaryDoubleTextWindow.new
	
	
	class ItemDataWindow < DoubleTextWindow
		LINE = 6
		def get_damage_type_caption(damage_types)
			caption = ""
			
			caption << "斬" if @item.damage_types.include?(DT_SLASH)
			caption << "突" if @item.damage_types.include?(DT_THRUST)
			caption << "打" if @item.damage_types.include?(DT_BLOW)
			caption << "熱" if @item.damage_types.include?(DT_HEAT)
			caption << "冷" if @item.damage_types.include?(DT_COLD)
			
			return caption
		end
	
		def make_surface(width = 200)
			super(width, LINE)
		end
		
		def update(item = nil)
			@item = item
			
			@texts.clear
			
			if @item then
				mastery = nil
				case @item.data
				when WeaponModel
					
					type = ""
					type << (@item.long_range ? "遠距離武器" : "近距離武器")
					type << (@item.double_handed ? " (両手)" : "")

					unless @item.unidentified? then
						@texts << [type, ""]
						if @item.data.is_a?(WandModel) then
							@texts << ["術威力", (@item.magic_power_rate * 10).to_i]
							@texts << ["術制御", (@item.magic_control_rate * 10).to_i]
						else
							@texts << ["攻撃属性", get_damage_type_caption(@item.damage_types)]
							@texts << ["致傷力", "#{(@item.damage_rate * 10).to_i}×#{@item.attack_number}回"]
						end
					end
					mastery = current_member.get_mastery(:weapon, @item.data_symbol)
	
				when ArmorModel
				
					@texts << ["体防具", ""]
					unless @item.unidentified? then
						@texts << ["防護力", @item.defense]
						@texts << ["回避補正", sprintf("%+d", @item.avoid_bonus)]
						@texts << ["耐性", ""]
					end
					mastery = current_member.get_mastery(:armor, @item.data_symbol)
				when ShieldModel
				
					@texts << [(@item.data.is_a?(LargeShieldModel) ? "大型の盾" : "小型の盾"), ""]
					unless @item.unidentified? then
						@texts << ["追加防護力", sprintf("%d", @item.extra_defense)]
					end
					mastery = current_member.get_mastery(:shield, @item.data_symbol)
				when ArmGuarderModel
				
					@texts << ["アクセサリー", ""]
					@texts << ["回避補正", sprintf("%+d", @item.avoid_bonus)]
				when AccessoryModel
				
					@texts << ["アクセサリー", ""]

				end
				
				unless @item.unidentified? then
					@item.extra_captions.each do |caption|
						@texts << [caption, ""]
					end
				end
				
				if mastery then
					(LINE - @texts.size - 1).times do
						@texts << []
					end
					@texts << ["習熟率", sprintf("%d%%", mastery)]
				end

			end
			
			super()
		end
	end
	ITEM_DATA_WINDOW = ItemDataWindow.new.make_surface
	
	class CharacterSpecWindow < DoubleTextWindow
		def make_surface(width = 200)
			super(width, 8)
		end
		
		def update(character = nil)
			
			@character = character
			@texts.clear
			if @character then
				#@texts << ["武器属性", "打"]
				@texts << ["致傷力", @character.weapon_power]
				@texts << ["攻撃回数", @character.weapon_attack_number]
				@texts << ["命中", @character.weapon_hitting]
				if @character.spellcaster? then
					@texts << ["術威力", @character.magic_power] 
					@texts << ["術制御", @character.magic_control]
				else
					@texts << [] << []
				end
				@texts << ["防護力", @character.defense]
				if @character.shield then
					@texts << [nil, "(盾 +#{@character.defense})"]
				else
					@texts << []
				end
				@texts << ["回避", @character.avoid]

			end
			super()
		end
	end
	
	
	class SelectableWindow < Window
		attr_accessor :index, :select_items, :right_texts
		attr_accessor :select_y, :select_height, :cursor_surface
		attr_accessor :display_max, :display_top
		attr_accessor :font, :right_font
		
		def initialize
			@index = 0
			@select_items = []
	
			@select_y = 0
			@select_height = REGULAR_FONT.line_skip + 6
			@display_max = nil
			@display_top = 0
			
			@font = REGULAR_FONT
			@right_font = REGULAR_FONT
	
			super
		end
		
		def display_bottom
			return(@display_max ? @display_top + @display_max - 1 : @select_items.size)
		end
		
	
		def make_surface(width = nil, line = nil)
			self.set_height_from_line(line) if line
			super(width, nil)
			@cursor_surface = SDL::Surface.new(SDL::SRCALPHA, @width - 1*2, @select_height, $screen)
			@cursor_surface.fill_rect(0, 0, @width, @height, ColorTable::SELECT)
			@cursor_surface.unlock
			@cursor_surface.set_alpha(SDL::SRCALPHA | SDL::RLEACCEL, 200)
			@cursor_surface.display_format
			
			@display_max = nil
			
			self.reset_index
			self.update
			return self
	
		end
		
		def set_height_from_line(line)
			@height = @select_y + PADDING * 2 + @select_height * line
		end


		def blit
			SDL.blit_surface2(@back_surface, nil, $screen, [@left, @top])
			SDL.blit_surface2(@cursor_surface, nil, $screen, [@left + 1, @top + PADDING + line_top(@index - @display_top)]) if @index && self == $windows.last
			#SDL.blit_surface2(@cursor_surface, nil, $screen, [@left + 1, @top + PADDING + @select_y + 2 + @select_height * (@index - @display_top)]) if self == $windows.last && @index
			SDL.blit_surface2(@front_surface, nil, $screen, [@left, @top])			
			return self
		end
		
		def update(select_items = nil)
			@select_items = select_items if select_items
		
			super()
			if @display_max then
				(@display_top..(self.display_bottom)).each do |i|
					draw_select_item_caption(i)
					
				end
			else
				(0...(@select_items.size)).each do |i|
					draw_select_item_caption(i)
				end
			end
			
			# 上下移動カーソル表示
			font = PAGE_CURSOR_FONT
			if self.display_up_cursor? then
				self.draw_text_right("▲", 0, 0, self.client_width, ColorTable::WHITE, font)
				
			end
			
			if self.display_down_cursor? then
				self.draw_text_right("▼", 0, self.client_height - font.text_size("▼")[1], self.client_width, ColorTable::WHITE, font)
			end
			
			return self
		end
		
		def line_top(line_index)
			return(@select_y + @select_height * line_index)
		end
		
		def line_text_top(line_index)
			return self.line_top(line_index) + ((@select_height - REGULAR_FONT.height) / 2).to_i
		end
		
		def draw_select_item_caption(index, color = nil)
			item = @select_items[index]
			return unless item
			if item.caption then
				left_color = (item.disabled? ? ColorTable::GRAY : item.left_color)
				caption = item.caption
				#caption = item.caption + self.get_shortcut_caption(item.keycode) if item.keycode
				self.draw_text_left(caption, 0, line_text_top(index - @display_top), left_color, @font)
			end
			
			if item.right_caption then
				right_color = (item.disabled? ? ColorTable::GRAY : item.right_color)

				self.draw_text_right(item.right_caption, 0, line_text_top(index - @display_top), self.client_width, right_color, @right_font)
			end

		end
		
		def get_shortcut_caption(keycode)
			return "(#{SDL::Key.get_key_name(keycode)})".upcase
		end
		
		def display_up_cursor?
			return(@display_top >= 1)
		end
		
		def display_down_cursor?
			return(self.display_bottom < (@select_items.size - 1))
		end
		
		def current_id
			return(self.current_item ? self.current_item.id : nil)
		end
		
		def current_item
			return(@index && @select_items[@index] ? @select_items[@index] : nil)
		end
		
		def set_index_from_id(id)
			@index = @select_items.map{|x| x.id if x}.index(id)
			return self
		end
		
		def on_enter_key
			
			if self.current_item && !(self.current_item.disabled?) then
				self.on_select(self.current_id)
			end

		end
		
		def on_change
			
			self.update if @display_max && @select_items.size > @display_max
			WARNING_WINDOW.update(current_item)
			$phase.on_change
		end
		
		def on_select(id)
			$phase.on_select(id)
		end
		
		
		
		def regulate_index
		
			# 選択可能なアイテムがなければ、index=0
			if @select_items.compact.empty? then
				@index = 0
				return self
			end
			
			@index ||= 0

			
			# 下方向にオーバーしていれば一番下の位置とする
			if @index >= @select_items.size then
				@index = (@select_items.size - 1)
			end
			
			# 上方向にオーバーしていれば一番上の位置とする
			if @index <= -1 then
				@index = 0
			end
			
			return self
		end

		# @indexが画面内に収まるように表示を調整
		def regulate_display
			#if Config::PAGE_SCROLL then
			#	self.regulate_page
			#else
				# カーソルが表示範囲より上にある
				if @index < @display_top then
					@display_top = [@index, @select_items.size - @display_max].min
				# カーソルが表示範囲より下にある
				elsif @index > self.display_bottom then
					@display_top = [(@index - @display_max + 1), 0].max
				end
			#end
		
			return self
		end
		
		def regulate_page
			self.page = (@index / @display_max).to_i
			return self
		end
		
		def up
			return if @select_items.compact.size <= 1
		
			@index ||= 0
			@index = prev_index(@index, @select_items.size)
			
			# SelectItemがなければスキップ
			while @select_items[@index].nil? do
				@index = prev_index(@index, @select_items.size)
			end
			
			self.regulate_display
			
		end
		
		def down
				
			return if @select_items.compact.size <= 1
			
			@index ||= 0
			@index = next_index(@index, @select_items.size)		
			
			while @select_items[@index].nil? do
				@index = next_index(@index, @select_items.size)
			end
	
			self.regulate_display
			
		end
		
		def page_up_shortcut
			if @display_max.nil? then
				@index = 0
				self.regulate_index
			elsif Config::PAGE_SCROLL then
				@index -= @display_max
				self.regulate_index.regulate_page
			else
				@display_top = [(@display_top -  (@display_max - 1)), 0].max
				@index = self.display_bottom
			end
			self.on_change
			
		end
		
		def page_down_shortcut
			if @display_max.nil? then
				@index = @select_items.size - 1
				self.regulate_index
			elsif Config::PAGE_SCROLL then
				@index += @display_max
				self.regulate_index.regulate_page
			else
				@display_top += (@display_max - 1)
				@display_top = @select_items.size - @display_max if self.display_bottom >= @select_items.size
				@index = @display_top
			end
			self.on_change

			
		end
		
		def page
			return (@display_top / @display_max).to_i
		end
		
		def page=(value)
			@display_top = @display_max * value
		end
		
		def page_top
			return(self.page * @display_max)
		end
		
		def page_max
			return((@select_items.size / @display_max).to_i + 1)
		end
		
		# マウスカーソルを選択肢の位置に合わせる
		def warp(index = @index)
			if $game_config.mouse_operation then
				SDL::Mouse.warp(@left + @width / 2, @top + @select_y + (@select_height * self.index_on_display) + @select_height / 2)
			end
			return self
		end
		
		def reset_index
			@index = 0
			@display_top = 0
			
			if @select_items.compact.size >= 1 then
				while @select_items[@index].nil? do
					@index = next_index(@index, @select_items.size)
				end
			end

			if @select_items.compact.find{|x| !(x.disabled?)} then
				while @select_items[@index].disabled? do
					@index = next_index(@index, @select_items.size)
				end
			end
			
			return self
		end
		
		def index_on_display
			return(@index - @display_top)
		end
		
		def get_mouse_index(x, y)
			return unless $game_config.mouse_operation
			
			
			top = @top + @select_y + PADDING + 2
			if @display_max then
				bottom = top + @select_height * [@select_items.size, @display_max].min
			else
				bottom = top + @select_height * @select_items.size
			end

			if ((@left)..(self.right)).include?(x) &&	(top...bottom).include?(y) then
				index = ((y - top) / @select_height) + @display_top
				return (@select_items[index] ? index : nil)
			else
				return nil
			end
		end
		
		def mouse_on_down_arrow?(x, y)
			return false
		
			client_right = self.right - PADDING
			client_bottom = self.bottom - PADDING
			font = REGULAR_FONT
			p font.text_size("▼")
			if ((client_right - font.text_size("▼")[0])..client_right).include?(x) &&
			((client_bottom - font.text_size("▼")[1])..client_bottom).include?(y) then
				return true
			end
		end
		
		# 選択肢を選ぶ。indexがnilなら選択解除
		def change(index)
			@index = index
			self.on_change if @index
			return self
		end
	
	end
	
	
	SELECT_WINDOW = SelectableWindow.new
	
	
	class TemporarySelectableWindow < SelectableWindow
		
	
		def on_select(id)
			throw(:exit_operation_loop, id)
		end
		
		def on_cancel
			throw(:exit_operation_loop, :cancel)
		end
		
		undef on_left_key
		undef on_right_key
		def on_key_press(keycode)
			return
		end
	end
	
	
	class SelectItem
		attr_accessor :id, :real_id, :caption, :keycode, :right_caption, :left_color, :right_color
		attr_accessor :enabled, :warning
			
		def initialize(id, caption, shortcut_key = nil, right_caption = nil)
			@id = id
			@real_id = id
			@caption = caption
			@right_caption = right_caption
			@left_color = ColorTable::WHITE
			@right_color = ColorTable::WHITE
			@enabled = true
			@warning = nil
			self.set_shortcut(shortcut_key) if shortcut_key
			
		end
		
		def set_shortcut(key)			
			if (0..9).include?(key.to_i) then
				@keycode = SDL::Key.const_get("K#{key.to_i}")
			elsif ('A'..'Z').include?(key) then
				@keycode = SDL::Key.const_get(key)
			else
				@keycode = key
			end
			
			return self
		end
		
		def disabled?
			return @id.nil? || !(enabled)
		end
		
		def enable
			@enabled = true
			return self
		end
		
		def disable(warning = nil)
			@enabled = false
			@warning = warning
			return self
		end
		
		

	end
	
	
	class ItemWindow < SelectableWindow
		def make_surface
			set_width
			super(nil, ITEM_MAX)
			
		end
		
		def set_width
			self.client_width = CHAR_WIDTH * (2 + 24)
		end
		
	
		def update(disp_shortcut = false)
			@select_items.clear
			member = current_member
			member.items.each_index do |i|
				item = member.items[i]
				if disp_shortcut then
					symbol = "#{(i+1) % 10}:"
				elsif member.equipped?(item) then
					symbol = 'E '
				else
					symbol = '  '
				end
				@select_items << SelectItem.new(i, symbol + item.name)
				@select_items.last.set_shortcut((i+1) % 10) if disp_shortcut
				set_right_captions
				@select_items.last.disable if check_disable(item)
			end
			
			super(nil)
		end
		
		def set_right_captions
		end
		
		def check_disable(item)
			return false
		end
	end
	
	class ItemSellWindow < ItemWindow
		def set_width
			self.client_width = CHAR_WIDTH * (2 + 24 + 6)
		end
		
		def set_right_captions
			@select_items.each do |select|
				select.right_caption = current_member.items[select.id].sell_price
			end
		end
	end

	class IdentificationWindow < ItemWindow
		def set_width
			self.client_width = CHAR_WIDTH * (2 + 24 + 6)
		end
		
		def set_right_captions
			@select_items.each do |select|
				select.right_caption = (current_member.items[select.id].unidentified? ? "3" : "")
			end
		end
		
		def check_disable(item)
			!(item.unidentified?) || current_member.gold < 3
		end

	end
	

	
	

	
	class MultiColumnWindow < SelectableWindow
		ITEM_NAME_WIDTH = REGULAR_FONT.text_size("＊＊＊＊＊＊＊＊＊＊＊＊")[0]
		attr_accessor :columns
	
		def initialize
			super
			@columns = []
			@select_y = TYPE_FONT.line_skip
		end
		
		def update
			super
			cx = 0
			@columns.each do |column|
	
				if column.align == AL_RIGHT then
					draw_text_right(column.header, cx, 0, column.width, ColorTable::WHITE, COLUMN_CAPTION_FONT)
				else
					draw_text_center(column.header, cx, 0, column.width, ColorTable::WHITE, COLUMN_CAPTION_FONT)
				end
				
				column.draw_captions(self, cx)
				
				cx = cx + column.width + column.right_margin
			end
			
			return self
		end
		
		def clear_captions
			@columns.each{|x| x.captions.clear}
		end
		
	end
	MULTI_COLUMN_WINDOW = MultiColumnWindow.new
	
	
	class ColumnItem
		attr_accessor :header, :width, :right_margin, :captions, :align
		
		def initialize(header, captions, width, align = AL_CENTER, right_margin = 30)
			@header = header
			@width = width
			@captions = captions
			@align = align
			
			@right_margin = right_margin
	
		end
		
		def draw_captions(window, left)
			if window.display_max then
				cps = []
				((window.display_top)..(window.display_bottom)).each do |i|
					cps << @captions[i]
				end
			else
				cps = @captions
			end
			
			cps.each_index do |i|
				caption = cps[i]
				if caption then
					case @align
					when AL_CENTER
						window.draw_text_center(caption, left, window.line_text_top(i), @width)
					when AL_LEFT
						window.draw_text_left(caption, left, window.line_text_top(i))
					when AL_RIGHT
						window.draw_text_right(caption, left, window.line_text_top(i), @width)
					end
				end
				
			end

		end
	end
	

	
	class MenuWindow < SelectableWindow
		attr_accessor :caption_window
		def initialize(line, menu_width = 240)
			super((SCREEN_WIDTH - menu_width) / 2, 100, menu_width, Window.get_line_window_height(line))
			@caption_window = TextWindow.new(@left, 60, @width, 1)
			@caption_window.align = AL_CENTER
			
		end
		
		def refresh
			super
			@caption_window.refresh
		end
	end
	
	class YNWindow < SelectableWindow
		attr_accessor :yes_proc, :no_proc
				
		def make_surface
			@select_items << SelectItem.new(:yes, "#{Lang::CommandName::Yes}(Y)", SDL::Key::Y)
			@select_items << SelectItem.new(:no, "#{Lang::CommandName::No}(N)", SDL::Key::N)

			super(120, 2)
		end

=begin
		def on_select(id)
			@proc ||= @yes_proc
			case id
			when :yes
				if @yes_proc then
					@proc.call
					self.hide
					@proc = nil
					@silent = false
				else
					$phase.on_enter_key
				end
			when :no
				self.hide
				SEQUENCIAL_MESSAGE_WINDOW.hide
			else
				LOGGER.error("エラー:YNWindowに未知のID #{id} が渡された")
				raise
			end					
		end
=end

		def on_select(id)
			case id
			when :yes
				throw(:exit_operation_loop, true)
			when :no
				throw(:exit_operation_loop, false)

			end
		end

		def on_cancel
			self.on_select(:no)
		end
		
	end
	YN_WINDOW = YNWindow.new.make_surface
	
	
	
	
	class MemberWindow < MultiColumnWindow
		attr_reader :hp_guage_surface, :hp_guage_warning_surface, :hp_guage_back_surface, :hp_guage_decrease_surface
		attr_reader :ct_guage_surface, :ct_guage_back_surface, :ct_guage_decrease_surface
		attr_reader :extra_columns, :member_ids
		def initialize
			super
			@extra_columns = []
			@member_ids = []
		end
		
		def make_surface(line)
			super(SCREEN_WIDTH - 8 * 2, line)
			@left = 8

			@hp_guage_surface = SDL::Surface.load("res/hp.png")
			@hp_guage_surface.display_format
			@hp_guage_warning_surface = SDL::Surface.load("res/hp_warning.png")
			@hp_guage_warning_surface.display_format
			@ct_guage_surface = SDL::Surface.load("res/ct.png")
			@ct_guage_surface.display_format


			@hp_guage_back_surface = SDL::Surface.new(0, @hp_guage_surface.w, @hp_guage_surface.h, $screen)
			@hp_guage_back_surface.fill_rect(0, 0, @hp_guage_surface.w, @hp_guage_surface.h, [40, 40, 40])
			@hp_guage_back_surface.display_format
			@hp_guage_decrease_surface = SDL::Surface.new(0, @hp_guage_surface.w, @hp_guage_surface.h, $screen)
			@hp_guage_decrease_surface.fill_rect(0, 0, @hp_guage_surface.w, @hp_guage_surface.h, [100, 40, 40])
			@hp_guage_decrease_surface.display_format
			@ct_guage_back_surface = SDL::Surface.new(0, @ct_guage_surface.w, @ct_guage_surface.h, $screen)
			@ct_guage_back_surface.fill_rect(0, 0, @ct_guage_surface.w, @ct_guage_surface.h, [40, 40, 40])
			@ct_guage_back_surface.display_format
			@ct_guage_decrease_surface = SDL::Surface.new(0, @ct_guage_surface.w, @ct_guage_surface.h, $screen)
			@ct_guage_decrease_surface.fill_rect(0, 0, @ct_guage_surface.w, @ct_guage_surface.h, [100, 40, 40])
			@ct_guage_decrease_surface.display_format


			return self
		end
		
		def members
			@member_ids.map{|x| $members[x]}
		end
		
		def viewable_members
			self.members[(@display_top)..(self.display_bottom)]
		end
		
		def update_items(member_ids = nil, extra_columns = nil)
			@member_ids = member_ids if member_ids
			members = self.members
			@columns.clear
			
			@columns << get_first_column
			@columns.last.right_margin = 0
			
			@columns << NameColumn.new("NAME", [], 100, AL_LEFT)
			
			@columns << ClassColumn.new("CLASS", members.map{|x| "#{x.class_initial}:L#{x.level}"}, 60, AL_CENTER)
			@columns << HPColumn.new("HP", [], 100, AL_LEFT, 20)
			
			if extra_columns then
				@extra_columns = extra_columns
			end
			
			if @extra_columns && @extra_columns.empty? then
				if $phase.is_a?(TreasurePhase) then
					@columns << ColumnItem.new("UNLOCK", members.map{|x| x.skill_levels[:unlock]}, 30, AL_RIGHT)
					@columns << ColumnItem.new("RM.TRAP", members.map{|x| x.skill_levels[:remove_trap]}, 30, AL_RIGHT)
				elsif $section == TOWN then
					@columns << ColumnItem.new("GOLD", members.map{|x| x.gold}, 60, AL_RIGHT)
				else
					@columns << ColumnItem.new("", nil, CHAR_WIDTH*3, AL_LEFT, 8)
					@columns.last.captions = members.map{|x| x.shield_usable? && $section == BATTLE ? $game_config.shield_mark : ""} 

					@columns << ColumnItem.new("CONDITION", members.map{|x| x.condition_caption}, 200, AL_CENTER)
				end
			else
				@columns += @extra_columns
			end
	
			@select_items.clear
			members.each_index do |i|
				@select_items << SelectItem.new(i, nil)
				
			end
			
			return self
		end
		
		def get_first_column
			ColumnItem.new("", [], CHAR_WIDTH * 2, AL_LEFT)
		end
		
		def show_shortcut
			@select_items.each_index do |i|
				@select_items[i].set_shortcut(i + 1)
				@columns.first.captions[i] = "#{i+1}:"
			end
			return self
		end
		

		
		
		def add_extra_item(id, caption, keycode = nil)
			@select_items << SelectItem.new(id, caption, keycode)
		end
		
	
		
		alias update_surface update
		
		def update(member_ids = nil, extra_columns = nil)
			self.update_items(member_ids, extra_columns)
			self.update_surface
			return self
		end
		
		
		
		class NameColumn < ColumnItem
			def draw_captions(window, left)
			
				

				members = window.viewable_members
				members.each_index do |i|
					if members[i].dead? then
						color = ColorTable::RED
					elsif $phase.is_a?(BattleRoundPhase) then
						if $current_action && members[i] == $current_action.actor then
							color = ColorTable::BLUE
						else
							color = ColorTable::WHITE
						end
					elsif window.select_items[i].nil? || window.select_items[i].disabled? then
						color = ColorTable::GRAY
					elsif i == $member_index then
						case $phase
						when MemberChangablePhase, BattleMemberCommandPhase
							color = ColorTable::BLUE
						else
							color = ColorTable::WHITE
						end
					else
						color = ColorTable::WHITE
					end
					
					window.draw_text_left(members[i].name, left, window.line_text_top(i), color)
				end
	
			end
			
		end
		
	
		class HPColumn < ColumnItem
			def draw_captions(window, left)
				members = window.viewable_members
				members.each_index do |i|
					member = members[i]
					text_top = window.line_text_top(i)
					guage_top = PADDING + window.line_top(i+1) - 2
					guage_width = (window.hp_guage_back_surface.w * (member.hp_max.to_f / member.hp_max_base.to_f)).to_i
					
					# 最大値表示
					SDL.blit_surface(window.hp_guage_back_surface, 0, 0, guage_width, 2, window.front_surface, left, guage_top)

					# 減少値＆現在値表示
					hp_rate = member.hp.to_f / member.hp_max.to_f
					last_hp_rate = member.last_hp.to_f / member.hp_max.to_f

					if hp_rate > 0 && member.alive? then
						SDL.blit_surface(window.hp_guage_decrease_surface, 0, 0, (last_hp_rate * guage_width).to_i, 2, window.front_surface, left, guage_top)

						surface = (hp_rate < 0.25 ? window.hp_guage_warning_surface : window.hp_guage_surface)
						SDL.blit_surface(surface, 0, 0, (hp_rate * guage_width).to_i, 2, window.front_surface, left, guage_top)
					end

					if member.dead? then
						window.draw_text_right("DEAD", left, text_top, 40, ColorTable::RED)
					elsif member.hp <= 0 then
						window.draw_text_right(member.hp, left, text_top, 40, ColorTable::RED)
					else
						window.draw_text_right(member.hp, left, text_top, 40)
					end
					
					window.draw_text_center("/", left + 40, text_top, 20)
					window.draw_text_right(member.hp_max, left + 40 + 20, text_top, 40, (member.hp_max_penalty > 0 ? [255, 160, 160] : ColorTable::WHITE))


				end
	
			end
		end
		
		
		class ClassColumn < ColumnItem
			def draw_captions(window, left)
				super
				members = window.viewable_members
				members.each_index do |i|
					member = members[i]
					next if member.is_a?(Enemy)
					text_top = window.line_text_top(i)
					guage_top = PADDING + window.line_top(i+1) - 2
					guage_width = window.ct_guage_back_surface.w
					guage_left = left + 3
					
					exp_rate = (member.exp - Member::EXP_TABLE[member.level]).to_f / Member::EXP_NEXT_TABLE[member.level+1].to_f
					exp_rate = 1.0 if exp_rate > 1.0
					window.front_surface.put(window.ct_guage_back_surface, guage_left, guage_top)
					SDL.blit_surface(window.ct_guage_surface, 0, 0, (exp_rate * guage_width).to_i, 2, window.front_surface, guage_left, guage_top)
	
	
				end

	
			end
		end


	end
	
	
	class PartyWindow < MemberWindow
		def make_surface
			super(MEMBER_MAX + 1)
			self.bottom = SCREEN_HEIGHT - 8
		end
		
		def update_items(extra_columns = nil)
			$party.update_formation
			super($party.member_ids, extra_columns)
			show_shortcut if $phase.is_a?(MemberChangablePhase)

		end
		
		def update(extra_columns = nil)
			self.update_items(extra_columns)
			self.update_surface
			return self
		end

		
		def get_first_column
			ColumnItem.new("", members.map{|x| x.position_caption}, CHAR_WIDTH * 2, AL_LEFT)
		end
		
		def on_select(id)
			throw(:exit_operation_loop, id)
		end
		
		def on_cancel
			throw(:exit_operation_loop, nil)
		end

	end
	
	PARTY_WINDOW = PartyWindow.new
	PARTY_WINDOW.make_surface
	
	class OtherMemberWindow < MemberWindow
		def make_surface(line)
			super(line)
			self.top = 8
			@display_max = line
			return self
		end
	end
	
	class EnemyWindow < SelectableWindow
		def make_surface
			super(PARTY_WINDOW.width, 4)
			@top = 8
			self.set_position(:center, nil)
			return self
		end
		
		def update_items
			@select_items.clear
			$troop.groups.each_index do |i|
				group = $troop.groups[i]
				next if group.empty?
				item = SelectItem.new(i, "#{i+1}:#{group.first.name} - #{group.size}匹")
				active = group.find_all{|x| x.active?}.size
				item.right_caption = "(行動可:#{active})" if active < group.size 
				item.set_shortcut(i+1)
				@select_items << item
			end
			return self
		end
		
		alias :update_surface :update
		
		def update
			self.update_items
			self.update_surface
			return self
		end
	end
	ENEMY_WINDOW = EnemyWindow.new.make_surface


	


	
	


	class ShopBuyListWindow < MultiColumnWindow
		BOW_MODEL = Weapon.new(:LongBow)
		AUTOBOW_MODEL = Weapon.new(:CrossBow)
		
		attr_reader :description_window, :name_window
		attr_reader :data_window, :help_window
		attr_accessor :products, :product_symbols, :product_type
		
		def initialize
			super
			@products = []
			@product_type = PD_WEAPON
		end
		
		
		def current_product
			return(@index ? @products[self.current_id] : nil)
		end
		
		
		
		def update(product_type = nil)
			@product_type = product_type if product_type
			@products = PRODUCTS[@product_type]
		
			
		
			# 商品リスト設定
			@columns.clear
			
			@columns << ColumnItem.new("NAME", [], 200, AL_LEFT)
			@columns.last.right_margin = 0
			@columns << ColumnItem.new("PRICE", [], 40, AL_RIGHT)
	
			@select_items.clear
			member = DarkHall.current_member
			@products.each_index do |i|
				product = @products[i]
			
				next unless product.usable_by?(member)
				@columns[1].captions << product.price
	
				@select_items << SelectItem.new(i, product.name)
				@select_items.last.disable if member.gold < product.price
			end
			
			super()
		end
		
	end
	
	
	class MasterableWindow < Window
		
		def make_surface
			self.set_height_from_line(5)
			super(300)
		end
		
		def update(member_class = Fighter)
			super()
			member = member_class.new

			name_list = %W(短剣 片手剣 両手剣 槍 騎兵槍 斧 棍棒 弓 自動弓 杖 小型盾 大型盾
			               外套 軽鎧 重鎧)
			class_list = [KnifeModel, SwordModel, LargeSwordModel,
			              SpearModel, LargeSpearModel, AxeModel, ClubModel, BowModel,
										AutoBowModel, WandModel, SmallShieldModel, LargeShieldModel,
										CloakModel, LightArmorModel, HeavyArmorModel]
			
			cx = 0
			cl = 0
			name_list.each_index do |i|
				draw_text_left(name_list[i], cx, line_text_top(cl), (class_list[i].new.usable_by?(member) ? ColorTable::WHITE : ColorTable::GRAY))
				if i.modulo(5) == 4 then
					cx += self.client_width / 3
					cl = 0
				else
					cl += 1
				end
			end
			
			return self
		end
	end
	
	class StatusWindow < Window
	
		attr_reader :item_list_left, :item_list_top, :page
		
		def initialize
			super
			@page = 1
		end
		
		def make_surface
			super(460, 360)
		end
		
		def get_grid_position(x, y)
			left = 4 + TT_FONT.text_size(' ')[0] * x
			right = 4 + TT_FONT.line_skip * y
			return left, top
		end
		
		
		def draw(text, x, y)
			left, top = get_grid_position(x, y)
			draw_text_left(text, left, top, ColorTable::WHITE, TT_FONT)
		end
		
		def update
			super()
			@member = $party.members[$member_index]

			cx = 0
			cy = 0
			case page
			when 1
				# 名前＆性別
				@name_window.update
				cy += 1
				return 
		
				# クラス
				self.draw_text_left("クラス", left_x, line_text_top(line))
				self.draw_text_right(@member.class_caption, left_x, line_text_top(line), left_width)
				line += 1
	
				# 年齢
				self.draw_text_left("年齢", left_x, line_text_top(line))
				self.draw_text_right(@member.age, left_x, line_text_top(line), left_width)
				line += 2
	
				
				# レベル
				self.draw_text_left("レベル", left_x, line_text_top(line))
				self.draw_text_right(@member.level, left_x, line_text_top(line), left_width)
				line += 1
		
				# 経験値
				self.draw_text_left("経験値", left_x, line_text_top(line))
				self.draw_text_right(@member.exp, left_x, line_text_top(line), left_width)
				line += 1		
		
				# 所持金
				self.draw_text_left("所持金", left_x, line_text_top(line))
				self.draw_text_right(@member.gold, left_x, line_text_top(line), left_width)
				line += 2	
		
		
				# 能力値
				
				
				ABILITY_CC_NAMES.each do |name|
					self.draw_text_left(Lang::AbilityName.const_get(name), left_x + 4, line_text_top(line))
					self.draw_text_right(@member.get_ability(name), left_x + 4, line_text_top(line), left_width)
					line += 1
				end
				
=begin
		
				# 呪文リスト
				if @member.spellcaster? then
					self.draw_text_left("呪文", cx, cy)
					
					@member.spells.each_index do |i|
						spell = @member.spells[i]
						x = cx + 16
						x += 100 if i.modulo(2) == 1
						y = cy + line_skip + (line_skip * (i / 2).to_i)
						
						name = SPELL_DATA[spell].short_name
						self.draw_text_left(sprintf("%s(%2d)", name, @member.mp[spell]), x, y)
					end
					
				end
		
				cy += (line_skip*5 + 8)
				
					
				# アイテムリスト
				self.draw_text_left("所持品", cx, cy)
				cx += 16
				cy += (line_skip + 8)
				@item_list_left = cx
				@item_list_top = cy
				
				@member.items.each_index do |i|
					case i
					when @member.weapon_index, @member.armor_index, 
					@member.accessory_indexes[0], @member.accessory_indexes[1]
						self.draw_text_left("E", cx, cy)
					end
					self.draw_text_left(@member.items[i].name, cx + 16, cy)
					cy += line_skip
				end
			when 2
			
				# 名前＆性別
				self.draw_text_left(@member.name, cx, cy)
				self.draw_text_right((@member.sextype == :Male ? "男性" : "女性"), cx, cy, 240)
				cy += line_skip * 2
		
				# クラス
				self.draw_text_left("クラス", cx, cy)
				self.draw_text_right(@member.class_caption, cx, cy, left_width)
				cy += line_skip
	
				# 年齢
				self.draw_text_left("年齢", cx, cy)
				self.draw_text_right(@member.age, cx, cy, left_width)
				cy += (line_skip + 8)
	
				
				# レベル
				self.draw_text_left("レベル", cx, cy)
				self.draw_text_right(@member.level, cx, cy, left_width)
				cy += line_skip
		
				# 経験値
				self.draw_text_left("経験値", cx, cy)
				self.draw_text_right(@member.exp, cx, cy, left_width)
				cy += line_skip
		
		
				# 所持金
				self.draw_text_left("所持金", cx, cy)
				self.draw_text_right("#{@member.gold}", cx, cy, left_width)
				cy += line_skip
		
		
				# 運勢
				self.draw_text_left("運勢", cx, cy)
				self.draw_text_right(@member.luck, cx, cy, left_width)
				cx += 20
				cy += (line_skip + 8)
=end
		

			end
			
			return self
		end
		
		def next_page
			@page += 1
			update
		end
		
		def prev_page
			@page -= 1
			update
		end
	end
	STATUS_WINDOW = StatusWindow.new

	
end
