#! ruby -Ku
# 基本スクリプト

$LOAD_PATH << './lib'
require 'inifile'
require 'ciphersaber'
require 'uconv'
require 'json'

require 'jcode'
require 'yaml/store'
require 'pathname'
require 'logger'
require 'benchmark'
require 'fileutils'

require 'sdl'
require 'fpstimer'
require 'pp'

require 'rubyplus'

begin
	module DarkHall
	
	
		module_function
		def put_loading_texts
			# 画面初期化
			$screen.fill_rect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, [ 0, 0, 0 ])
	
			cy = 16
			LOADING_TEXTS.each do |text|
				REGULAR_FONT.draw_blended_utf8($screen, text, 16, cy, 255, 255, 255)
				cy += (REGULAR_FONT.line_skip + 4)
			end
			TITLE_FONT.draw_blended_utf8($screen, "Dark Hall", 400 + 4, 300 + 4, 60, 60, 60)
			TITLE_FONT.draw_blended_utf8($screen, "Dark Hall", 400, 300, 255, 255, 255)
	
			$screen.flip
		end
		
		TIMELOG_FORMAT = "%s: %.5f sec"
		def timelog(title = "Time", indent_level = 1)
			log = "\t" * indent_level + sprintf(TIMELOG_FORMAT, title, Benchmark.realtime{yield})
			LOGGER.puts(log)
		end
		

		CS_PASSWORD = 'darkhall'
		def encrypt(str)
			return(str.cs_encrypt(CS_PASSWORD))
		end
			
		def decrypt(str)
			return(str.cs_decrypt(CS_PASSWORD))
		end
		

		require 'const'
		require 'config'
		require 'lang'
		require 'font'
		require 'class_logger'

		SDL.init(SDL::INIT_VIDEO | SDL::INIT_AUDIO | SDL::INIT_JOYSTICK)
	
		$screen = SDL::set_video_mode(SCREEN_WIDTH, SCREEN_HEIGHT, 16, (Config::DISABLE_HARDWARE_DRAW ? SDL::SWSURFACE : SDL::HWSURFACE))
		LOGGER.log(VideoInitializedLog)
		
		SDL::WM.set_caption("DarkHall (alpha test - period4 fix3)", "DarkHall")
	

		
	
		
		LOADING_TEXTS = ["画面の初期化を完了しました"]
		LOADING_TEXTS << "音楽デバイスを初期化しています..."
		put_loading_texts
	
		timelog {
			require 'sound'
			LOGGER.info("音楽デバイス初期化成功")
		}
		LOADING_TEXTS.last.concat("完了.")
		LOADING_TEXTS << "各種データを読み込んでいます..."
		put_loading_texts
	
		# キー初期化
		$joysticks = []
	
		(0...(SDL::Joystick.num)).each do |i|
			$joysticks[i] = SDL::Joystick.open(i)
		end
		LOGGER.log(JoystickInitializedLog)

	
		SDL::Key.enable_key_repeat(240, 120)
		
		
		
		
		
	end
	
	require 'class_savedata_new'
	require 'class'
	require 'class_character'
	require 'class_action'
	require 'class_battle'
	require 'class_party'
	require 'class_troop'
	require 'class_dungeon'
	require 'class_state'
	require 'class_effect'
	require 'class_switch'
	require 'class_game_config'


	require 'data_item'
	require 'data_equipment'
	require 'data_enemy'
	require 'data_spell'

	require 'message'
	require 'roll'
	require 'roll_table'
	require 'class_dialog'
	
	require 'class_phase'
	
	
	
	module DarkHall
		

	
		def text_width(text, font = REGULAR_FONT)
			return REGULAR_FONT.text_size(text)[0]
		end

		def text_height(text, font = REGULAR_FONT)
			return REGULAR_FONT.text_size(text)[1]
		end
	
		def current_window
			return $windows.last
		end
		
		def current_member
			return $party.members[$member_index]
		end
		
		# obsolete
		def party_title
			return $party.name
		end
		
		def random_get(array)
			if array.empty? then
				return nil
			else
				return array[rand(array.size)]
			end
		end
		
		def eval_message(message, tags = {})
			message = "" unless message
			tags.each_pair do |tagname, replace|
				if replace then
					message = message.gsub(/\%#{tagname}/, replace)
				end
			end
			return message
		end
		
		def next_index(index, max)
			index += 1
			index = 0 if index >= max
			return index
		end
		
		def prev_index(index, max)
			index -= 1
			index = (max - 1) if index <= -1
			return index
		end
		
		def fold_in_range(value, range)
			if range.include?(value) then
				return value
			elsif value < range.first then
				return range.first
			else
				return range.last
			end	
		end
		
		
		
		
		def show_message(message, &enter_proc)
			SEQUENCIAL_MESSAGE_WINDOW.centering(false).update(message).show
			SEQUENCIAL_MESSAGE_WINDOW.proc = enter_proc
		end
		
		def message(*msgs)
			msgs.flatten!
			upside = msgs.shift
			downsides = msgs
			
			
			if $section == Section::BATTLE then
				MESSAGE_WINDOW.set_position(:center, 16)
			else
				MESSAGE_WINDOW.centering(false)
			end
			yield(MESSAGE_WINDOW) if block_given?

			if downsides.empty? then
				MESSAGE_WINDOW.update(upside).show
				operation_loop
				MESSAGE_WINDOW.hide

			else
				line_max = (upside.count("\n") + 1) + downsides.map{|x| x.count("\n") + 1}.max
		
				downsides.each do |downside|
					msg = (downside.empty? ? upside.dup : (upside + "\n" + downside))
		
					if (line = msg.count("\n") + 1) < line_max then
						msg.concat("\n " * (line_max - line))
						
					end
					MESSAGE_WINDOW.update(msg).show
					operation_loop("MessageLoop")
				end
				MESSAGE_WINDOW.hide

			end
		end
		
		def dungeon_message(*msgs)
			message(*msgs){|window|
				window.set_position(:center, DUNGEON_WINDOW.bottom - 24)
			}
		end
		
		def show_debug_data_window(texts = [[]])
			DEBUG_DATA_WINDOW.make_surface(240, texts.size).update(texts).centering(false).show
			operation_loop
			DEBUG_DATA_WINDOW.hide
		end
		
		
		def ask(message, default_yes = true)
			SEQUENCIAL_MESSAGE_WINDOW.centering(false).update(message).show
			if default_yes
				YN_WINDOW.index = 0
			else
				YN_WINDOW.index = 1
			end

			YN_WINDOW.dock_beside(SEQUENCIAL_MESSAGE_WINDOW, :bottom).set_position(:center, YN_WINDOW.top)
			YN_WINDOW.warp.show
			
			answer = operation_loop("AskLoop")
			YN_WINDOW.hide
			SEQUENCIAL_MESSAGE_WINDOW.hide
			return answer
		end
		
		def select(select_items)
			window = TemporarySelectableWindow.new
			width = select_items.map{|x| window.font.text_size(x.caption)[0]}.max + 16
			window.make_surface(width, select_items.size).update(select_items).centering
			
			yield(window) if block_given?
			window.regulate_index.show.warp
			answer = operation_loop
			window.hide
			
			return answer
		end
		
		
		def member_select(caption, default_member_index = nil)
			INFORMATION_WINDOW.make_surface(160).align = AL_LEFT
			INFORMATION_WINDOW.update(caption).dock_beside(PARTY_WINDOW, :top).show

			if default_member_index then
				PARTY_WINDOW.index = default_member_index
			else
				PARTY_WINDOW.reset_index
			end
			PARTY_WINDOW.show_shortcut.update_surface
			PARTY_WINDOW.show.warp

			selected = operation_loop
			INFORMATION_WINDOW.hide
			PARTY_WINDOW.update
			$windows.delete(PARTY_WINDOW)
			$windows.unshift(PARTY_WINDOW)
			return selected
		end
		
		def battle(troop)
			$troop = troop
			Phase.change(BattleStartPhase.new)
			result = operation_loop("BattleLoop")
			return result
		end
		
		def show_system_menu(key)
			return nil
			title_window = TextWindow.new
			title_window.text = "システムメニュー"
			menu_window = SelectableWindow.new
			menu_window.select_items = [
				SelectItem.new(nil, "操作説明(F1)", SDL::Key::F1),
				SelectItem.new(:config, "ゲーム設定(F2)", SDL::Key::F2),
				SelectItem.new(:quit, "終了(F4)", SDL::Key::F4),
				SelectItem.new(:cancel, "キャンセル")
			]
			title_window.make_surface(200, 1).set_position(8, 8).show
			menu_window.make_surface(title_window.width, 4)
			menu_window.index = (menu_window.select_items.map{|x| x.keycode}.index(key) || 0)
			menu_window.dock_beside(title_window, :bottom, -1).warp.show
			
			def menu_window.on_select(id)
				case id
				when :config
					Phase.change(GameConfig.new)
				when :quit
					exit(0)
				when :cancel
					self.on_cancel
				end
			end
			
			def menu_window.on_cancel
				$windows[-1].hide
				$windows[-1].hide
			end
		end
		
		
		def get_alive_members
			return $party.members.find_all{|x| x.alive?}
		end

		
		def refresh
			
			TIMER.wait_frame do
				DarkHall.draw_screen
			end
		end
		
		def wait(frame)
			draw_screen
			frame.times do
				TIMER.wait_frame do
				end
			end
			
			# イベントキューを空っぽにする
			while SDL::Event2.poll do
			end
		end
		
		def draw_screen
	
			# カーソル点滅処理
			if Config::ENABLE_ALPHA_EFFECT then
				if $cursor_alpha_increasing then
					$cursor_alpha += 5
				else
					$cursor_alpha -= 5
				end
				
				if $cursor_alpha >= 255 then
					$cursor_alpha = 255
					$cursor_alpha_increasing = false
				elsif $cursor_alpha <= 160 then
					$cursor_alpha = 160
					$cursor_alpha_increasing = true
				end
			end
			
			# 画面再描画
			$screen.fill_rect( 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, [ 0, 0, 0 ] )
			if Config::ENABLE_ALPHA_EFFECT && current_window.is_a?(SelectableWindow) then
				$windows.last.cursor_surface.set_alpha(SDL::SRCALPHA, $cursor_alpha)
			elsif Config::ENABLE_ALPHA_EFFECT && $phase.is_a?(DungeonMapPhase) then
				$windows.last.cursor_surface.set_alpha(SDL::SRCALPHA, $cursor_alpha)
				$windows.last.range_cursor_surface.set_alpha(SDL::SRCALPHA, $cursor_alpha)
			end
			$windows.each do |window|
				window.blit
				
			end
				
			# FPS表示
			COLUMN_CAPTION_FONT.draw_blended_utf8($screen, "FPS:#{TIMER.real_fps}", 600, 4, 255, 255, 255) if DISP_FPS

	
			$screen.flip
		end
		
		def flash
			$screen.fill_rect( 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, [ 255, 255, 255 ] )
			$screen.flip

			
		end
		
		def operation_loop(title = "NamelessLoop")
			$operation_loop_stack << title
			result = catch(:exit_operation_loop){
				loop do
					while event = SDL::Event2.poll
						current_window = DarkHall.current_window
						case event
						when SDL::Event2::Quit
							exit
						when SDL::Event2::KeyDown
							# ショートカット処理
							if current_window.is_a?(SelectableWindow)  then
								index = current_window.select_items.map{|x| x.keycode if x}.index(event.sym)
								if index then
									current_window.change(index)
									refresh
									refresh
									unless current_window.current_item.disabled? then
										SE.select
									 	current_window.on_select(current_window.select_items[index].id)
									end
								end
							end
				
						
							case event.sym
							when SDL::Key::F1, SDL::Key::F2, SDL::Key::F3, SDL::Key::F4
								DarkHall.show_system_menu(event.sym)
							when SDL::Key::UP
								Input.up
				
							when SDL::Key::DOWN
								Input.down
				
							when SDL::Key::LEFT
								Input.left
							when SDL::Key::RIGHT
								Input.right
				
							when SDL::Key::PAGEUP
								Input.page_up
				
							when SDL::Key::PAGEDOWN
								Input.page_down
				
							when SDL::Key::HOME
								Input.home
							when SDL::Key::END
								Input.end
							when SDL::Key::LSHIFT, SDL::Key::RSHIFT
								Input.shift_down
							when SDL::Key::LCTRL, SDL::Key::RCTRL
								Input.ctrl_down
				
							when SDL::Key::RETURN, SDL::Key::Z, SDL::Key::SPACE
								Input.enter
							when SDL::Key::ESCAPE, SDL::Key::X, SDL::Key::BACKSPACE
								Input.esc
								
							
							else
								current_window.on_key_press(event.sym)
							end
							
						when SDL::Event2::KeyUp
							case event.sym
							when SDL::Key::LSHIFT, SDL::Key::RSHIFT
								Input.shift_up
							when SDL::Key::LCTRL, SDL::Key::RCTRL
								Input.ctrl_up
							end
							
						when SDL::Event2::MouseMotion
							if $game_config.mouse_operation then

								# マウスが移動したかどうかの判定
								if ((event.xrel.abs + event.yrel.abs) >= 1) then
									if current_window.is_a?(SelectableWindow) then
										index = current_window.get_mouse_index(event.x, event.y)
										if index && index != current_window.index then
											current_window.change(index)
										end
									end
								end
							end
							
						when SDL::Event2::MouseButtonDown
							if $game_config.mouse_operation then
								case event.button
								when SDL::Mouse::BUTTON_LEFT
									Input.enter
								when SDL::Mouse::BUTTON_RIGHT
									Input.esc
								end
							end
						when SDL::Event2::JoyButtonDown
							case event.button
							when $game_config.key_assign[:Decide]
								Input.enter
							when $game_config.key_assign[:Cancel]
								Input.esc
							else
								if event.button == $game_config.key_assign[:LeftWalk] && $phase.on_left_walk_key then
									Input.left_walk
								elsif event.button == $game_config.key_assign[:RightWalk] && $phase.on_right_walk_key then
									Input.right_walk
								elsif event.button == $game_config.key_assign[:PageUp] then
									Input.page_up
								elsif event.button == $game_config.key_assign[:PageDown] then
									Input.page_down
								end
							end
						when SDL::Event2::JoyAxis
							case event.axis
							when 0
								if event.value >= 20000 then
									Input.right
								elsif event.value <= -20000 then
									Input.left
								end
							when 1
								if event.value >= 20000 then
									Input.down
								elsif event.value <= -20000 then
									Input.up
								end
							end
						end
						
					end
					
					refresh
				
				end
			}
			$operation_loop_stack.pop
			return result
		end




		
		module Input
			module_function
			
			def alt_pressed?
				return (SDL::Key.press?(SDL::Key::LALT) || SDL::Key.press?(SDL::Key::RALT))
			end
			
			def ctrl_pressed?
				return (SDL::Key.press?(SDL::Key::LCTRL) || SDL::Key.press?(SDL::Key::RCTRL))
			end
			
			def shift_pressed?
				return (SDL::Key.press?(SDL::Key::LSHIFT) || SDL::Key.press?(SDL::Key::RSHIFT))
			end
			
			def page_move_shifted?
				SDL::Key.scan
				if alt_pressed? || ctrl_pressed? then
					return true
				end
				
				if $game_config.key_assign[:PageMoveShift] then
					$joysticks.each do |stick|
						return true if stick.button($game_config.key_assign[:PageMoveShift])
					end
				end
				
				return false
			
			end
			
			def side_walk_shifted?
				SDL::Key.scan
				if alt_pressed? || ctrl_pressed? then
					return true
				end
				
				if $game_config.key_assign[:SideWalkShift] then
					$joysticks.each do |stick|
						return true if stick.button($game_config.key_assign[:SideWalkShift])
					end
				end
				
				return false
			end
			
			def enter
				$phase.select_se
				current_window.on_enter_key
			end
			
			def esc
				$phase.cancel_se
				current_window.on_cancel
			end
			
			def up
				if defined?($phase.on_up_key) && !defined?(current_window.up) then
					$phase.on_up_key
				elsif page_move_shifted? then
					page_up
				elsif defined?(current_window.up) then
					SE.cursor_move
					current_window.up
					current_window.on_change
				else
					current_window.on_key_press(SDL::Key::UP)
				end
			
			end
			
			def down
				if defined?($phase.on_down_key) && !defined?(current_window.down) then
					$phase.on_down_key
				elsif page_move_shifted? then
					page_down
				elsif defined?(current_window.down) then
					SE.cursor_move
					current_window.down
					current_window.on_change
				else
					current_window.on_key_press(SDL::Key::DOWN)
				end
			end
			
			def left
				if defined?(current_window.on_left_key) then
					current_window.on_left_key
				else
					current_window.on_key_press(SDL::Key::LEFT)
				end
			end
			
			def right
				if defined?(current_window.on_right_key) then
					current_window.on_right_key
				else
					current_window.on_key_press(SDL::Key::RIGHT)
				end
			end
			
			def left_walk
				$phase.on_left_walk_key
			end
			
			def right_walk
				$phase.on_right_walk_key
			end
			
			def shift_up
				current_window.on_shift_up
			end
			
			def shift_down
				current_window.on_shift_down
			end

			def ctrl_up
				current_window.on_ctrl_up
			end
			
			def ctrl_down
				current_window.on_ctrl_down
			end

			def page_up
				if defined?(current_window.page_up_shortcut)
					SE.cursor_move
					current_window.page_up_shortcut
				end
			end
			
			def page_down
				if defined?(current_window.page_down_shortcut)
					SE.cursor_move
					current_window.page_down_shortcut
				end
	
			end
			
			def home
				if current_window.is_a?(SelectableWindow) then
					SE.cursor_move
					current_window.change(0).regulate_display.update
				end
			end
			
			def end
				raise($operation_loop_stack.join("/"))
				if current_window.is_a?(SelectableWindow) then
					SE.cursor_move
					current_window.change(current_window.select_items.size - 1).regulate_display.update
				end
			end
			
			
		end
	
	end
	
	
	
	
	
	
	include DarkHall
	
	
	# 定数
	FIND_PRODUCT_TABLE = {}
	FIND_PRODUCT_TABLE[PD_WEAPON] = []
	
	ABILITY_NAMES = %W(Str Tec Agl Vit Int Wil Psy Cha)
	
	start = 0
	symbols = [:IronKnife, :LongSword, :ShortBow]
	symbols.each do |symbol|
		FIND_PRODUCT_TABLE[PD_WEAPON].fill(Weapon.new(symbol), start, 4)
		start += 4
	end
	
	def set_find_products
		$find_products = {}
		$find_products[PD_WEAPON] = []
		$find_products[PD_ARMOR] = []
		$find_products[PD_GOODS] = []
		
		# 武器の掘り出し物
		products = $find_products[PD_WEAPON]
		(0..1).each do |rarity|
			4.times do
				products << FIND_PRODUCT_TABLE[PD_WEAPON][rand(FIND_PRODUCT_TABLE.size)].dup.set_prefix(rarity)
			end
		end
		
		
		
	end
	
	
	

	
	$party = Party.new
	$members = []
	$dungeon_parties = []
	$bar_member_ids = []
	$reviving_member_ids = []
	$revived_member_ids = []
	$lost_member_ids = []
	$switch = Switch.new
	$game_config = GameConfig.new
	Dungeon.load_data
	$found_area = []
	$found_area[1] = Array.new(FLOORS[1].width).fill(0)
	$found_area[1].each_index{|i| $found_area[1][i] = Array.new(FLOORS[1].height).fill(0)}
	$room_troops = []
	$room_troops[1] = []
	
	def DarkHall.set_room_troops(floor_id = 1)
		$room_troops[floor_id].clear
		FLOORS[floor_id].room_max.times do |id|
			troop = RoomTroop.new
			region = FLOORS[floor_id].room_to_region_table[id]
			beast = (region == :Maze ? :Rizardman : :Kobolt)
			
			case region
			when :ShutterSwitchRoom, :BossRoom
				next
			else
				case rand(10)
				when 5,6
					troop.add_enemies(beast, rand(3) + 1)
				when 7,8
					troop.add_enemies(:Kobolt, rand(3) + 2)
					troop.add_enemies(:Kobolt, rand(3) + 1)
				when 9
					troop.add_enemies(beast, rand(3) + 1)
					troop.add_enemies(:Slime, rand(4) + 1)
				else
					next
				end
			end
			
			$room_troops[floor_id][id] = troop
				
		end
		
		LOGGER.log(SetRoomTroopLog, 1)
	end
	

	$week = 1
	$round = 0
	
	$find_products = {}
	set_find_products
	
=begin
	$party.members = [Fighter.new.entry(Member::MALE), HeavyFighter.new.entry(Member::MALE), Magician.new.entry(Member::FEMALE)]
	$party.members[0].items << Weapon.new(:LongSword)
	$party.members[0].weapon_index = 0
	$party.members[0].items << Armor.new(:LeatherArmor)
	$party.members[0].armor_index = 1
	$party.members[0].items << Weapon.new(:SteelClaymore)
	$party.members[0].items << Item.new(:Bandage)
	$party.members[0].items << Item.new(:RedElixir)
	$party.members[0].items << Weapon.new(:LongBow)
	$party.members[0].items.last.prefix = :good
=end
	
	


	
	
	
	
	
	LOADING_TEXTS.last.concat("完了.")
	LOADING_TEXTS << "サーフィスを初期化しています..."
	DarkHall.put_loading_texts
	
	require 'graphic'
	
	$windows = []
	$selectable_windows = []
	$operation_loop_stack = []
	require 'class_window'
	
	
	
	
	
	LOADING_TEXTS.last.concat("完了.")
	DarkHall.put_loading_texts
	

	#puts FLOORS[1].area_data.map{|x| x.inspect}
	
	DarkHall.set_room_troops(1)

	GC.start
	
	if Config::DATA_SELECTABLE then
		Phase.change(TitlePhase.new)
	else
		load_profile(:n)
		Phase.change(TownMenuPhase.new)

	end
	Dungeon.load_data

	
	
	
	$member_index = 0
	PARTY_WINDOW.update
	

	
	
	TIMER = FPSTimerLight.new(Config::FPS)
	TIMER.reset
	LOGGER.info("タイマーを#{Config::FPS}FPSで初期化")
	
	$cursor_alpha = 255
	$cursor_alpha_increasing = false
	

=begin
	LOGGER.puts "敵データ"
	enemies = ENEMY_DATA.keys.map{|x| Enemy.new(x)}
	enemies.each do |enemy|
		LOGGER.puts enemy.name
		LOGGER.puts "\t致傷力:#{enemy.weapon_power}"
		LOGGER.puts "\t命中:#{enemy.weapon_hitting}"
		LOGGER.puts "\t回避:#{enemy.avoid}"
		LOGGER.puts "\t速度:#{enemy.speed}"
	end
=end

	DarkHall.operation_loop("FirstLoop")
	
	raise StandardError, "first loop stopped"
#rescue
#	LOGGER.fatal "#{$!.message}"
#	$!.backtrace.each{|x| LOGGER.fatal "\t#{x}"}
end
	
