# 音管理

module DarkHall
	
	SDL::Mixer.open(SDL::Mixer::DEFAULT_FREQUENCY, SDL::Mixer::DEFAULT_FORMAT, 2, 1024)
	
	class BGM
		attr_reader :path
		
		def initialize(key)
			if Config::INI['BGMFilePath'][key] then
				@path = Pathname.new(Config::INI['BGMFilePath'][key])
				@path = Pathname.new('bgm/') + @path
			end
			@music = nil
		end
	
		def load
			# リロードはなし
			unless @music then
				LOGGER.warn "BGM #{@path} はすでにロード済み"
			end
			
			begin
				if @path.exist? && @path.readable? then
					@music = SDL::Mixer::Music.load(@path.to_s)
					LOGGER.info "BGM #{@path} のロードに成功"
				else
					LOGGER.warn "BGM #{@path} は存在しないか、ロードできない" if @path
				end
				
			rescue SDL::Error
				LOGGER.error "BGM #{@path} のオープンに失敗 (#{$!.message})"
			end
			return self
		end
		
		def unload
			@music = nil
			LOGGER.info "BGM #{@path} のインスタンスを破棄"

			return self
		end
		
		def loaded?
			return !(@music.nil?)
		end
		
		def play
			if @music then
				$current_bgm = self
				SDL::Mixer.playMusic(@music,-1)
				LOGGER.info "BGM #{@path} を演奏開始"
			end
			return self
		end
		
		def BGM.change(to_bgm)
			return if $current_bgm.eql?(to_bgm)
			BGM.halt
			$current_bgm.unload if $current_bgm
			to_bgm.load unless to_bgm.loaded?
			to_bgm.play
		end
		
		def BGM.halt
			SDL::Mixer.halt_music
			$current_bgm = nil
		end
		
		def BGM.fade_out
			SDL::Mixer.fade_out_music(1500)
			$current_bgm = nil
		end
		
		
		TOWN = BGM.new('Town')
		BATTLE = BGM.new('Battle')
		BOSS_BATTLE = BGM.new('BossBattle')
		CAMP = BGM.new('Camp')
		DARK_HALL = BGM.new('DarkHall') || BGM.new('Dungeon')
		
		SDL::Mixer.set_volume_music(Config::BGM_VOLUME)
		
		
	end
	
	module SE
		CURSOR_MOVE_SE = SDL::Mixer::Wave.load('se/cursor_move.wav')
		SELECT_SE = SDL::Mixer::Wave.load('se/select.wav')
		CANCEL_SE = SDL::Mixer::Wave.load('se/kizi020.wav')
		EQUIP_SE = SDL::Mixer::Wave.load('se/eucalyptus_tree/katana1.wav')
		ENCOUNT_SE = SDL::Mixer::Wave.load('se/eucalyptus_tree/ect4.wav')
		COIN_SE = SDL::Mixer::Wave.load('se/osa/bell04.wav')
		STEP_SE = SDL::Mixer::Wave.load('se/osa/step05.wav')
		DOOR_SE = SDL::Mixer::Wave.load('se/osa/door01.wav')
		SONAR_SE = SDL::Mixer::Wave.load('se/osa/cursor34.wav')
		REMOVE_TRAP_SE = SDL::Mixer::Wave.load('se/osa/chari13_a.wav')
		UNLOCK_SE = SDL::Mixer::Wave.load('se/osa/chari13_b.wav')
		TRAP_SWITCH_SE = SDL::Mixer::Wave.load('se/osa/kachi05.wav')
		OPEN_BOX_SE = SDL::Mixer::Wave.load('se/osa/open45.wav')
		CHANNEL_1 = 2
		CHANNEL_2 = 3
		CHANNEL_3 = 4
		STEP_CHANNEL = 5
		SDL::Mixer.set_volume(CHANNEL_1, Config::SE_VOLUME)
		SDL::Mixer.set_volume(CHANNEL_2, Config::SE_VOLUME)
		SDL::Mixer.set_volume(CHANNEL_3, Config::SE_VOLUME)
		SDL::Mixer.set_volume(STEP_CHANNEL, 64)
		@channel = CHANNEL_1

		def SE.flip_channel
			case @channel
			when CHANNEL_1
				@channel = CHANNEL_2
			when CHANNEL_2
				@channel = CHANNEL_3
			when CHANNEL_3
				@channel = CHANNEL_1
			end
		end

		def SE.cursor_move
			SDL::Mixer.play_channel(@channel, CURSOR_MOVE_SE, 0)
			SE.flip_channel
		end
		
		def SE.select
			SDL::Mixer.play_channel(@channel, SELECT_SE, 0)
			SE.flip_channel
		end
	
		def SE.cancel
			SDL::Mixer.play_channel(@channel, CANCEL_SE, 0)
			SE.flip_channel
		end
		
		def SE.step
			SDL::Mixer.play_channel(STEP_CHANNEL, STEP_SE, 0)
			SE.flip_channel
		end
		
		def SE.door
			SDL::Mixer.play_channel(@channel, DOOR_SE, 0)
			SE.flip_channel
		end
	
	
	
		def SE.equip
			SDL::Mixer.play_channel(@channel, EQUIP_SE, 0)
			SE.flip_channel
		end
		
		def SE.coin
			SDL::Mixer.play_channel(@channel, COIN_SE, 0)
			SE.flip_channel
		end
		
		def SE.encount
			SDL::Mixer.play_channel(@channel, ENCOUNT_SE, 0)
			SE.flip_channel
		end
		
		def SE.remove_trap
			SDL::Mixer.play_channel(@channel, REMOVE_TRAP_SE, 0)
			SE.flip_channel
		end

		def SE.unlock
			SDL::Mixer.play_channel(@channel, UNLOCK_SE, 0)
			SE.flip_channel
		end
		
		def SE.trap_switch
			SDL::Mixer.play_channel(@channel, TRAP_SWITCH_SE, 0)
			SE.flip_channel
		end


		def SE.open_box
			SDL::Mixer.play_channel(@channel, OPEN_BOX_SE, 0)
			SE.flip_channel
		end

	
	end

end


